import logging
import os
import uuid

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.parameter as ParameterModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)
        return

    def shouldAllowEdit(self, value=None):
        if value is not None:
            self._shouldAllowEdit = value
        if not hasattr(self, '_shouldAllowEdit'):
            self._shouldAllowEdit = False
        return self._shouldAllowEdit

    def definition(self, value=None):
        if value is not None:
            self._definition = value
        if not hasattr(self, '_definition'):
            self._definition = False
        return self._definition

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        if self.shouldAllowEdit():
            self.setWindowTitle('Edit definition')
        else:
            self.setWindowTitle('View definition')

        comboBoxExecutableType = widget.comboBoxExecutableType 
        executableTypes = contextManager.getExecutableTypes()
        comboBoxExecutableType.addItems(executableTypes)

        # update all gui items with the values from the definition
        pomsetContext = self.pomsetContext()
        definition = self.definition()

        textEditName = widget.textEditName
        textEditName.setText(definition.name() or '')
        textEditDescription =widget.textEditDescription
        textEditDescription.setText(definition.description() or '')
        
        executable = definition.executable()
        executableType = contextManager.getTypeForExecutable(executable)
        index = comboBoxExecutableType.findText(executableType)
        if index >= 0:
            comboBoxExecutableType.setCurrentIndex(index)

        self.updateGuiForEditingExecutableType(executableType)

        self.populateParameters()

        if not self.shouldAllowEdit():
            comboBoxExecutableType.setEditable(False)
            comboBoxExecutableType.setEnabled(False)
            textEditName.setReadOnly(True)
            textEditDescription.setReadOnly(True)
        return


    def updateGuiForEditingExecutableType(self, executableType):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()


        executable = definition.executable()
        if executable is None or \
           not contextManager.getTypeForExecutable(executable) == executableType:
            executableClass = contextManager.getClassForExecutableType(executableType)
            executable = executableClass()


        visibilityValues = []
        if executableType == 'Shell':
            # turn off unnecessary widgets and
            # turn on necessary widgets
            visibilityValues = [True, True,
                                True,
                                False, False,
                                False, False,
                                True, True]

            widget.textEditExecutablePath.setText(os.path.sep.join(executable.path()))
            widget.checkBoxStageable.setChecked(executable.stageable())
            widget.textEditStaticArgs.setText(' '.join(executable.staticArgs()))
            pass
        elif executableType == 'Hadoop Jar':
            # turn off unnecessary widgets
            # turn on necessary widgets
            visibilityValues = [False, False,
                                False,
                                True, True,
                                True, True,
                                False, False]
            widget.textEditJarFile.setText(' '.join(executable.jarFile()))
            widget.textEditJarClass.setText(' '.join(executable.jarClass()))
            widget.textEditStaticArgs.setText(' '.join(executable.staticArgs()))
            pass
        elif executableType == 'Hadoop Streaming':
            # turn off unnecessary widgets
            # turn on necessary widgets
            visibilityValues = [False, False,
                                False,
                                False, False,
                                False, False,
                                False, False]
            pass
        elif executableType == 'Hadoop Pipes':
            # turn off unnecessary widgets
            # turn on necessary widgets

            visibilityValues = [True, True,
                                False,
                                False, False,
                                False, False,
                                False, False]
            widget.textEditExecutablePath.setText(os.path.sep.join(executable.pipesFile()))
            pass
        elif executableType == 'Python Eval':
            visibilityValues = [True, True,
                                False,
                                False, False,
                                False, False,
                                False, False]
            delimiter = '.'
            if not all([len(x) for x in executable.path()]):
                delimiter = ''
            widget.textEditExecutablePath.setText(delimiter.join(executable.path()))
            pass

        childWidgets = [widget.labelExecutablePath,
                        widget.textEditExecutablePath,
                        widget.checkBoxStageable,
                        widget.labelJarFile,
                        widget.textEditJarFile,
                        widget.labelJarClass,
                        widget.textEditJarClass,
                        widget.labelStaticArgs,
                        widget.textEditStaticArgs]
        for visibilityValue, childWidget in zip(visibilityValues,
                                                childWidgets):
            childWidget.setVisible(visibilityValue)
            pass

        if not self.shouldAllowEdit():
            widget.textEditExecutablePath.setReadOnly(True)
            widget.checkBoxStageable.setEnabled(False)
            widget.textEditJarFile.setReadOnly(True)
            widget.textEditJarClass.setReadOnly(True)
            widget.textEditStaticArgs.setReadOnly(True)

        return


    def populateParameters(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()

        tableParameters = widget.tableParameters
        while tableParameters.rowCount():
            tableParameters.removeRow(0)

        tableParameters.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        tableParameters.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        tableParameters.setColumnWidth(0, 200)
        tableParameters.setColumnWidth(1, 120)

        parametersTable = definition.parametersTable()
        filter = RelationalModule.ColumnValueFilter(
            'parameter',
            FilterModule.ObjectKeyMatchesFilter(
                filter = FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_DATA),
                keyFunction=lambda x: x.portType()
            )
        )
        parameters = RelationalModule.Table.reduceRetrieve(
            parametersTable,
            filter,
            ['parameter'],
            []
        )

        tableParameters.setRowCount(len(parameters))
        for rowIndex, parameter in enumerate(parameters):
            name = parameter.id() or ''
            item = QtGui.QTableWidgetItem('name')
            item.setData(Qt.UserRole, parameter)
            item.setData(Qt.DisplayRole, name)
            tableParameters.setItem(rowIndex, 0, item)

            direction = parameter.portDirection()
            if parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT):
                direction = 'output'
            item = QtGui.QTableWidgetItem('direction')
            item.setData(Qt.UserRole, parameter)
            item.setData(Qt.DisplayRole, direction)
            tableParameters.setItem(rowIndex, 1, item)
            pass

        return


    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.ApplyRole:
            if self.shouldAllowEdit():
                self.saveValues()
            pass
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            if self.shouldAllowEdit():
                self.saveValues()
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return

    @pyqtSlot(QModelIndex)
    def on_tableParameters_doubleClicked(self, index):

        parameter = index.data(Qt.UserRole).toPyObject()
        self.displayParameter(parameter)

        return


    @pyqtSlot(str)
    def on_comboBoxExecutableType_currentIndexChanged(self, value):
        try:
            self.updateGuiForEditingExecutableType(value)
        except TypeError, e:
            logging.debug('error on updating gui >> %s' % e)
        return

    @pyqtSlot()
    def on_buttonAdd_clicked(self):

        if not self.shouldAllowEdit():
            return

        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()

        name = '_'.join(['parameter', uuid.uuid4().hex[:3]])

        builder = contextManager.pomsetBuilder()

        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT
            }

        builder.addPomsetParameter(definition, name, attributes)

        self.populateParameters()

        return


    @pyqtSlot()
    def on_buttonEdit_clicked(self):
        contextManager = self.contextManager()

        tableParameters = self.widget().tableParameters
        parameters = list(set([x.data(Qt.UserRole).toPyObject() 
                               for x in tableParameters.selectedItems()]))

        if len(parameters) is 0:
            return

        parameterToEdit = parameters[0]

        self.displayParameter(parameterToEdit)

        return

    def displayParameter(self, parameter):

        contextManager = self.contextManager()

        # import the modules
        import pomsets_app.gui.qt.edit_parameter.widget as WidgetModule
        import pomsets_app.gui.qt.edit_parameter.controller as ControllerModule

        # initialize the widget
        controller = ControllerModule.Controller(self)
        controller.shouldAllowEdit(self.shouldAllowEdit())
        controller.contextManager(contextManager)
        controller.pomsetContext(self.pomsetContext())
        controller.parameter(parameter)
        controller.definition(self.definition())

        ui = WidgetModule.Ui_Dialog()
        ui.setupUi(controller)

        controller.widget(ui)

        # populate the dialog
        controller.populate()

        QObject.connect(contextManager.mainWindow(), 
                        SIGNAL("OnPomsetModified(PyQt_PyObject)"),
                        self.populateParameters)

        controller.show()

        return

    @pyqtSlot()
    def on_buttonRemove_clicked(self):

        if not self.shouldAllowEdit():
            return

        contextManager = self.contextManager()
        builder = contextManager.pomsetBuilder()
        pomsetContext = self.pomsetContext()
        pomset = self.definition()

        tableParameters = self.widget().tableParameters
        parameters = list(set([x.data(Qt.UserRole).toPyObject() 
                               for x in tableParameters.selectedItems()]))

        if len(parameters):
            for parameter in parameters:
                builder.removePomsetParameter(pomset, parameter.id())
            self.populateParameters()
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)

        return

    def saveValues(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()
        
        hasBeenModified = False
        
        name = str(widget.textEditName.text())
        if not name == definition.name():
            definition.name(name)
            hasBeenModified = True

        description = str(widget.textEditDescription.text())
        if not description == definition.description():
            definition.description(description)
            hasBeenModified = True


        executableType = str(widget.comboBoxExecutableType.currentText())
        executable = definition.executable()
        currentExecutableType = contextManager.getTypeForExecutable(executable)
        if not executableType == currentExecutableType:
            executableClass = contextManager.getClassForExecutableType(executableType)
            executable = executableClass()
            definition.executable(executable)
            hasBeenModified = True
            
        if executableType == 'Shell':
            executablePath = str(widget.textEditExecutablePath.text()).split(os.path.sep)
            if not executablePath == executable.path():
                executable.path(executablePath)
                hasBeenModified = True

            isStageable = widget.checkBoxStageable.isChecked()
            if not executable.stageable() == isStageable:
                executable.stageable(isStageable)
                hasBeenModified = True

            staticArgs = str(widget.textEditStaticArgs.text()).split(' ')
            if not executable.staticArgs() == staticArgs:
                executable.staticArgs(staticArgs)
                hasBeenModified = True

        elif executableType == 'Hadoop Jar':

            jarFile = str(widget.textEditJarFile.text()).split(' ')
            if not executable.jarFile() == jarFile:
                executable.jarFile(jarFile)
                hasBeenModified = True

            jarClass = str(widget.textEditJarClass.text()).split(' ')
            if not executable.jarClass() == jarClass:
                executable.jarClass(jarClass)
                hasBeenModified = True
            pass

        elif executableType == 'Hadoop Streaming':
            executable.jarFile([contextManager.getHadoopStreamingJar()])
            executable.jarClass([])
            pass

        elif executableType == 'Hadoop Pipes':

            pipesFile = str(widget.textEditExecutablePath.text()).split(os.path.sep)
            if not executable.pipesFile() == pipesFile:
                executable.pipesFile(pipesFile)
                hasBeenModified = True
            pass
        elif executableType == 'Python Eval':
            executablePath = widget.textEditExecutablePath.text().rsplit('.', 1)
            if len(executablePath) == 1:
                executablePath.insert(0, '')
            if not executablePath == executable.path():
                executable.path(executablePath)
                hasBeenModified = True

        if hasBeenModified:

            pomsetContext.isModified(True)
            
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)
        return

    # END class Controller
    pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/edit task dialog.ui'
#
# Created: Sat Apr 17 00:14:48 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(450, 456)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(250, 410, 181, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.labelName = QtGui.QLabel(Dialog)
        self.labelName.setGeometry(QtCore.QRect(20, 30, 62, 16))
        self.labelName.setObjectName("labelName")
        self.labelComment = QtGui.QLabel(Dialog)
        self.labelComment.setGeometry(QtCore.QRect(20, 70, 71, 16))
        self.labelComment.setObjectName("labelComment")
        self.labelParameters = QtGui.QLabel(Dialog)
        self.labelParameters.setGeometry(QtCore.QRect(20, 150, 91, 16))
        self.labelParameters.setObjectName("labelParameters")
        self.tableParameters = QtGui.QTableWidget(Dialog)
        self.tableParameters.setGeometry(QtCore.QRect(20, 180, 411, 171))
        self.tableParameters.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tableParameters.setObjectName("tableParameters")
        self.tableParameters.setColumnCount(4)
        self.tableParameters.setRowCount(0)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(1, item)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(2, item)
        item = QtGui.QTableWidgetItem()
        self.tableParameters.setHorizontalHeaderItem(3, item)
        self.textEditName = QtGui.QLineEdit(Dialog)
        self.textEditName.setGeometry(QtCore.QRect(100, 21, 331, 31))
        self.textEditName.setObjectName("textEditName")
        self.textEditComment = QtGui.QLineEdit(Dialog)
        self.textEditComment.setGeometry(QtCore.QRect(100, 60, 331, 31))
        self.textEditComment.setObjectName("textEditComment")
        self.buttonChangeGroup = QtGui.QPushButton(Dialog)
        self.buttonChangeGroup.setGeometry(QtCore.QRect(20, 360, 121, 32))
        self.buttonChangeGroup.setObjectName("buttonChangeGroup")
        self.checkBoxCritical = QtGui.QCheckBox(Dialog)
        self.checkBoxCritical.setGeometry(QtCore.QRect(20, 110, 87, 20))
        self.checkBoxCritical.setObjectName("checkBoxCritical")
        self.buttonToggleStaging = QtGui.QPushButton(Dialog)
        self.buttonToggleStaging.setGeometry(QtCore.QRect(150, 360, 141, 32))
        self.buttonToggleStaging.setObjectName("buttonToggleStaging")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Edit task", None, QtGui.QApplication.UnicodeUTF8))
        self.labelName.setText(QtGui.QApplication.translate("Dialog", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.labelComment.setText(QtGui.QApplication.translate("Dialog", "Comment", None, QtGui.QApplication.UnicodeUTF8))
        self.labelParameters.setText(QtGui.QApplication.translate("Dialog", "Parameters", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(0).setText(QtGui.QApplication.translate("Dialog", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(1).setText(QtGui.QApplication.translate("Dialog", "Stage file", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(2).setText(QtGui.QApplication.translate("Dialog", "Sweep", None, QtGui.QApplication.UnicodeUTF8))
        self.tableParameters.horizontalHeaderItem(3).setText(QtGui.QApplication.translate("Dialog", "Parameter sweep group", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonChangeGroup.setText(QtGui.QApplication.translate("Dialog", "Change group", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBoxCritical.setText(QtGui.QApplication.translate("Dialog", "Critical", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonToggleStaging.setText(QtGui.QApplication.translate("Dialog", "Toggle staging", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




import logging

import pomsets_app.gui.qt.application as ApplicationModule


def configLogging():
    """
    this will be called by all the main functions 
    to use the default setup for logging
    """
    # define a basic logger to write to file
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s %(levelname)-8s %(message)s',
                        datefmt='%a, %d %b %Y %H:%M:%S',
                        filename='/tmp/pomsets_app.log',
                        filemode='w')

    # end def configureLogging
    pass


def main():

    configLogging()
    app = ApplicationModule.Application()
    app.runProgram()

    return

if __name__=="__main__":
    main()




from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.library as LibraryModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)
        return

    def canvasPosition(self, value=None):
        if value is not None:
            self._canvasPosition = value
        if not hasattr(self, '_canvasPosition'):
            self._canvasPosition = None
        return self._canvasPosition

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        persistentLibrary = contextManager.persistentLibrary()
        transientLibrary = contextManager.transientLibrary()

        treeWidgetDefinitions = widget.treeWidgetDefinitions
        treeWidgetDefinitions.setHeaderHidden(True)


        notBootstrapFilter = FilterModule.constructNotFilter()
        notBootstrapFilter.addFilter(
            LibraryModule.getBootstrapLoaderPomsetsFilter())

        for library, filter, header in [(persistentLibrary, notBootstrapFilter, 'Library definitions'),
                                        (transientLibrary, FilterModule.TRUE_FILTER, 'Loaded definitions')]:
            rootItem = QtGui.QTreeWidgetItem([header])
            rootItem.setData(0, Qt.UserRole, None)
            treeWidgetDefinitions.addTopLevelItem(rootItem)
            definitions = RelationalModule.Table.reduceRetrieve(
                library.definitionTable(), 
                filter, ['definition'], [])
            for definition in definitions:
                item = QtGui.QTreeWidgetItem([definition.name() or 'anonymous'])
                item.setData(0, Qt.UserRole, definition)
                rootItem.addChild(item)
                pass
            pass
        return


    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            try:
                self.createNode()
            except ValueError, e:
                # this happens if a root item is selected

                # TODO:
                # should display an error dialog

                return
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return

    def createNode(self):

        widget = self.widget()
        treeWidgetDefinitions = widget.treeWidgetDefinitions
        
        selectedItems = treeWidgetDefinitions.selectedItems()
        data = [y for y in 
                [x.data(0, Qt.UserRole).toPyObject() for x in selectedItems]
                 if y is not None]
        if len(data) is 0:
            raise ValueError('need to select a definition to reference')
        
        name = str(widget.textEditName.text())
        if not name or not len(name):
            raise ValueError('need to specify name for new node')

        self.contextManager().mainWindow().OnCreateNode(
            self.pomsetContext(), self.pomsetReference(),
            data[0], self.canvasPosition(),
            name=name)
        
        return


    # END class Controller
    pass



from __future__ import with_statement

import simplejson as ConfigModule
import os
import pickle
import time
import tarfile
import user


def getDefaultConfigDir():
    # wx.StandardPaths.Get().GetUserDataDir() 
    # Return the directory for the user-dependent application data files: 
    # $HOME/.appname under Unix, 
    # c:/Documents and Settings/username/Application Data/appname under Windows and 
    # ~/Library/Application Support/appname under Mac

    # for now, we just use $HOME/.pomsets
    configDir = os.path.join(user.home, '.pomsets')
    return configDir


def getDefaultConfigPath(configDir=None):

    if configDir is None:
        configDir = getDefaultConfigDir()

    configPath = os.path.join(configDir, 'config')
    
    return configPath


def saveConfig(configData, configPath=None):

    # this is the default path
    if configPath is None:
        configPath = getDefaultConfigPath()

    with open(configPath, 'w') as f:
        ConfigModule.dump(configData, f, sort_keys=True, indent=4)
        pass
    return


def loadConfig(configPath=None):

    # this is the default path
    if configPath is None:
        configPath = getDefaultConfigPath()

    with open(configPath) as f:
        config = ConfigModule.load(f)
        return config

    raise IOError('could not read load config from config path %s' % configPath)


def createNewConfigDataObject():
    return {}


def backupLibrary(libraryDir, destination=None):

    if destination is None:
        destination = libraryDir + "." + str(int(time.time())) + ".tgz"
        
    tarball = tarfile.open(name=destination, mode='w:gz')
    tarball.add(libraryDir, arcname="library")
    tarball.close()

    return destination



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/license agreement.ui'
#
# Created: Mon Mar  8 11:11:43 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(541, 453)
        self.comboBoxLicense = QtGui.QComboBox(Dialog)
        self.comboBoxLicense.setGeometry(QtCore.QRect(20, 40, 281, 31))
        self.comboBoxLicense.setObjectName("comboBoxLicense")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(20, 20, 171, 16))
        self.label.setObjectName("label")
        self.textEditLicense = QtGui.QTextEdit(Dialog)
        self.textEditLicense.setGeometry(QtCore.QRect(20, 80, 501, 291))
        self.textEditLicense.setObjectName("textEditLicense")
        self.checkBoxRead = QtGui.QCheckBox(Dialog)
        self.checkBoxRead.setGeometry(QtCore.QRect(30, 380, 261, 20))
        self.checkBoxRead.setObjectName("checkBoxRead")
        self.buttonAgree = QtGui.QPushButton(Dialog)
        self.buttonAgree.setGeometry(QtCore.QRect(400, 410, 115, 32))
        self.buttonAgree.setObjectName("buttonAgree")
        self.buttonDisagree = QtGui.QPushButton(Dialog)
        self.buttonDisagree.setGeometry(QtCore.QRect(280, 410, 115, 32))
        self.buttonDisagree.setObjectName("buttonDisagree")

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "License agreement", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "Please choose a license:", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBoxRead.setText(QtGui.QApplication.translate("Dialog", "I have read the terms of the license", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonAgree.setText(QtGui.QApplication.translate("Dialog", "I agree", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonDisagree.setText(QtGui.QApplication.translate("Dialog", "I disagree", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/hadoop configuration dialog.ui'
#
# Created: Thu Mar  4 21:37:13 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(560, 180)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(190, 130, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(20, 29, 111, 16))
        self.label.setObjectName("label")
        self.textEditHadoopHome = QtGui.QLineEdit(Dialog)
        self.textEditHadoopHome.setGeometry(QtCore.QRect(140, 20, 391, 31))
        self.textEditHadoopHome.setObjectName("textEditHadoopHome")
        self.label_2 = QtGui.QLabel(Dialog)
        self.label_2.setGeometry(QtCore.QRect(20, 69, 101, 16))
        self.label_2.setObjectName("label_2")
        self.textEditStreamingJar = QtGui.QLineEdit(Dialog)
        self.textEditStreamingJar.setGeometry(QtCore.QRect(140, 60, 301, 31))
        self.textEditStreamingJar.setObjectName("textEditStreamingJar")
        self.buttonBrowse = QtGui.QPushButton(Dialog)
        self.buttonBrowse.setGeometry(QtCore.QRect(444, 60, 91, 32))
        self.buttonBrowse.setObjectName("buttonBrowse")
        self.checkBoxRelative = QtGui.QCheckBox(Dialog)
        self.checkBoxRelative.setGeometry(QtCore.QRect(150, 100, 231, 20))
        self.checkBoxRelative.setObjectName("checkBoxRelative")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Hadoop configuration", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "HADOOP_HOME", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "Streaming jar", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonBrowse.setText(QtGui.QApplication.translate("Dialog", "Browse", None, QtGui.QApplication.UnicodeUTF8))
        self.checkBoxRelative.setText(QtGui.QApplication.translate("Dialog", "Relative to HADOOP_HOME", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

class PomsetTreeWidget(QtGui.QTreeWidget):

    def __init__(self, *args, **kwds):
        QtGui.QTreeWidget.__init__(self, *args, **kwds)
        self.setColumnCount(1)
        
        # hide the header
        self.setHeaderLabels([''])
        self.setHeaderHidden(True)

        # enable the contextual menu
        self.setContextMenuPolicy( Qt.CustomContextMenu )
        self.connect(self, SIGNAL('customContextMenuRequested(const QPoint&)'), self.OnContextMenu)

        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)

        return

    def shouldEnableDragToCanvas(self, value=None):
        if value is not None:
            self._shouldEnableDragToCanvas = value
        if not hasattr(self, '_shouldEnableDragToCanvas'):
            self._shouldEnableDragToCanvas = False
        return self._shouldEnableDragToCanvas


    def mouseDoubleClickCallback(self, value=None):
        if value is not None:
            self._mouseDoubleClickCallback = value
        if not hasattr(self, '_mouseDoubleClickCallback'):
            self._mouseDoubleClickCallback = None
        return self._mouseDoubleClickCallback


    def shouldShowInternals(self, value=None):
        if value is not None:
            self._shouldShowInternals = value
        if not hasattr(self, '_shouldShowInternals'):
            self._shouldShowInternals = False
        return self._shouldShowInternals


    def contextualMenuClass(self, value=None):
        if value is not None:
            self._contextualMenuClass = value
        if not hasattr(self, '_contextualMenuClass'):
            self._contextualMenuClass = False
        return self._contextualMenuClass


    def shouldIndicateModified(self, value=None):
        if value is not None:
            self._shouldIndicateModified = value
        if not hasattr(self, '_shouldIndicateModified'):
            self._shouldIndicateModified = False
        return self._shouldIndicateModified

    def dragSelection(self, value=None):
        if value is not None:
            self._dragSelection = value
        if not hasattr(self, '_dragSelection'):
            self._dragSelection = (None, None)
        return self._dragSelection


    def shouldExpand(self, value=None):
        if value is not None:
            self._shouldExpand = value
        if not hasattr(self, '_shouldExpand'):
            self._shouldExpand = False
        return self._shouldExpand


    def pomsetLibrary(self, value=None):
        if value is not None:
            self._pomsetLibrary = value
        if not hasattr(self, '_pomsetLibrary'):
            self._pomsetLibrary = None
        return self._pomsetLibrary

    def pomsetFilter(self, value=None):
        if value is not None:
            self._pomsetFilter = value
        if not hasattr(self, '_pomsetFilter'):
            self._pomsetFilter = FilterModule.TRUE_FILTER
        return self._pomsetFilter

    def selectedPomsetContext(self, value=None):
        if value is not None:
            self._selectedPomsetContext = value
        if not hasattr(self, '_selectedPomsetContext'):
            self._selectedPomsetContext = None
        return self._selectedPomsetContext

    def contextManager(self, value=None):
        if value is not None:
            self._contextManager = value
        if not hasattr(self, '_contextManager'):
            self._contextManager = None
        return self._contextManager

    def emptyDataSourceString(self, value=None):
        if value is not None:
            self._emptyDataSourceString = value
        if not hasattr(self, '_emptyDataSourceString'):
            self._emptyDataSourceString = 'None'
        return self._emptyDataSourceString


    def addTreeNode(self, parentWidget, pomsetContext, 
                    name = None,
                    node = None, 
                    recursive=False):

        if name is None and node is not None:
            name = node.name()
            if not name or not len(name):
                name = 'Anonymous'

        isRoot = parentWidget is self
        if isRoot and self.shouldIndicateModified() and \
                pomsetContext is not None and \
                pomsetContext.isModified():
            name = name + ' (modified)'
            pass

        item = QtGui.QTreeWidgetItem([name])
        item.setData(0, Qt.UserRole, (pomsetContext, node))
        if isRoot:
            parentWidget.addTopLevelItem(item)
        else:
            parentWidget.addChild(item)

        if node:
            definitionToReference = node.definitionToReference()
            if recursive and not definitionToReference.isAtomic():
                for childNode in definitionToReference.nodes():
                    self.addTreeNode(item, pomsetContext, 
                                   name = childNode.name(),
                                   node = childNode,
                                   recursive=recursive)

        return

    def populate(self):
        self.clear()

        definitionTable = self.pomsetLibrary().definitionTable()
        pomsetContexts = RelationalModule.Table.reduceRetrieve(
            definitionTable,
            self.pomsetFilter(), ['context'], [])
        if len(pomsetContexts) is 0:
            # TODO:
            # add an empty item
            # with label being self.emptyDataSourceString()
            self.addTreeNode(self, None, name=self.emptyDataSourceString(),
                             node=None, recursive=False)
            pass
        else:
            for pomsetContext in pomsetContexts:

                # add pomsetContext to tree view
                # also check if self.shouldShowInternals()
                # if so, then recursively add 
                self.addTreeNode(self, pomsetContext,
                                 node=pomsetContext.reference(),
                                 recursive=self.shouldShowInternals())
                pass
            pass

        # TODO:
        # set the selection
        self.selectPomsetContext(self.selectedPomsetContext())
            
        if self.shouldExpand():
            self.expandAll()
        return



    def selectPomsetContext(self, pomsetContext):
        self.selectedPomsetContext(pomsetContext)
        return


    def mouseDoubleClickEvent(self, event):
        cb = self.mouseDoubleClickCallback()
        if cb is not None:
            cb(event)
        return


    def displayPomset(self, event, frame=None):
        item = self.itemAt(event.pos())
        if item:
            pomsetContext, pomsetReference = \
                item.data(0, Qt.UserRole).toPyObject()
            pomset = pomsetReference.definitionToReference()
            if pomsetContext is not None and not pomset.isAtomic():
                frame.app().contextManager().activePomsetContext(pomsetContext)

                # determine the path
                # using the tree widget items
                currentItem = item
                itemPath = [currentItem]
                while True:
                    parent = currentItem.parent()
                    if not isinstance(parent, QtGui.QTreeWidgetItem):
                        break
                    currentItem = parent
                    itemPath.insert(0, currentItem)
                    pass

                # extract the pomset path
                pomsetReferencePath = [
                    x.data(0, Qt.UserRole).toPyObject()[1]
                    for x in itemPath]

                frame.displayPomset(pomsetContext, pomsetReferencePath)
                frame.canvas.OnRefresh()
        return


    def mousePressEvent(self, event):

        buttons = event.buttons()
        keyModifiers = event.modifiers()
        if buttons == Qt.LeftButton:

            if self.shouldEnableDragToCanvas():
                item = self.itemAt(event.pos())
                if item:
                    pomsetContext, pomset = \
                        item.data(0, Qt.UserRole).toPyObject()
                    if pomsetContext is not None:
                        self.dragSelection((pomsetContext, pomset))
                pass

            # need to do this
            # because the tree view will not select something
            # if you're just clicking.  you have to 
            # move the mouse so that it has a rect to select
            self.setSelection(QRect(event.pos(), QSize(1,1)),
                              QtGui.QItemSelectionModel.SelectCurrent)

            pass

        event.ignore()
        return


    def mouseReleaseEvent(self, event):

        if self.shouldEnableDragToCanvas():
            self.OnDragToCanvas(event)

        # reset the drag selection
        self._dragSelection = (None, None)

        return


    def OnDragToCanvas(self, event):
        """
        NOTE:
        event.pos() is relative to the widget
        canvas.contentRect() is relative the canvas
        """

        pomsetContext, pomsetReference = self.dragSelection()
        if pomsetReference is None:
            return
        pomset = pomsetReference.definitionToReference()

        contextManager = self.contextManager()

        # TODO:
        # instead of doing nothing,
        # should query the user to see if they want to create a new context
        # and if so, continue 

        # TODO:
        # this check here should be extracted into something like
        # if contextManager.hasActivePomsetContext() and
        #      not contextManager.hasTaskForPomset(activePomsetContext.reference())
        displayedPomsetContext, displayedPomsetReferencePath, displayedIsEditable = \
            contextManager.currentDisplayedPomsetInfo()
        print "current displayed pomset info >> %s" % str(contextManager.currentDisplayedPomsetInfo())


        if displayedPomsetContext is None:
            # there's no active pomset to modify
            return

        if not displayedIsEditable:
            # not editable, so should not add
            return

        if contextManager.hasTaskForPomsetPath(displayedPomsetReferencePath):
            # execution has already started, so should not add
            return

        displayedPomsetReference = displayedPomsetReferencePath[-1]

        canvas = contextManager.canvas
        canvasLocalRect = canvas.contentsRect()

        # compute the canvas' position globally
        canvasScreenPosition = canvas.mapToGlobal(QPoint(0,0))
        canvasScreenRect = canvasLocalRect.translated(canvasScreenPosition)

        eventLocalPos = event.pos()
        eventScreenPos = self.mapToGlobal(eventLocalPos)
        eventCanvasPos = canvas.mapFromGlobal(eventScreenPos)
        
        if canvasScreenRect.contains(eventScreenPos):

            self.emit(
                SIGNAL("OnCreateNode(PyQt_PyObject,PyQt_PyObject,PyQt_PyObject,QPoint)"), 
                displayedPomsetContext, displayedPomsetReference, 
                pomset, eventCanvasPos)

            pass

        return


    def OnContextMenu(self, point):
        
        menuClass = self.contextualMenuClass()
        popupMenu = menuClass(self)

        item = self.itemAt(point)
        if item:
            pomsetContext, pomset = \
                item.data(0, Qt.UserRole).toPyObject()
            popupMenu.selection((pomsetContext, pomset))
        else:
            popupMenu.selection((None, None))

        popupMenu.position(point)
        popupMenu.contextManager(self.contextManager())

        popupMenu.bindEvents()
        popupMenu.popup(self.mapToGlobal(point))

        return


    def OnRefresh(self, *args, **kwds):
        self.populate()
        return

    # END class PomsetTreeWidget
    pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/euca2ools configuration dialog.ui'
#
# Created: Thu Mar  4 16:04:59 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(562, 283)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(180, 230, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.label = QtGui.QLabel(Dialog)
        self.label.setGeometry(QtCore.QRect(30, 30, 62, 16))
        self.label.setObjectName("label")
        self.label_2 = QtGui.QLabel(Dialog)
        self.label_2.setGeometry(QtCore.QRect(30, 70, 81, 16))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtGui.QLabel(Dialog)
        self.label_3.setGeometry(QtCore.QRect(30, 110, 71, 16))
        self.label_3.setObjectName("label_3")
        self.label_4 = QtGui.QLabel(Dialog)
        self.label_4.setGeometry(QtCore.QRect(30, 150, 62, 16))
        self.label_4.setObjectName("label_4")
        self.label_5 = QtGui.QLabel(Dialog)
        self.label_5.setGeometry(QtCore.QRect(30, 190, 81, 16))
        self.label_5.setObjectName("label_5")
        self.buttonBrowse = QtGui.QPushButton(Dialog)
        self.buttonBrowse.setGeometry(QtCore.QRect(444, 180, 91, 32))
        self.buttonBrowse.setObjectName("buttonBrowse")
        self.textEditURL = QtGui.QLineEdit(Dialog)
        self.textEditURL.setGeometry(QtCore.QRect(120, 21, 411, 31))
        self.textEditURL.setObjectName("textEditURL")
        self.textEditAccessKey = QtGui.QLineEdit(Dialog)
        self.textEditAccessKey.setGeometry(QtCore.QRect(120, 60, 411, 31))
        self.textEditAccessKey.setObjectName("textEditAccessKey")
        self.textEditSecretKey = QtGui.QLineEdit(Dialog)
        self.textEditSecretKey.setGeometry(QtCore.QRect(120, 100, 411, 31))
        self.textEditSecretKey.setObjectName("textEditSecretKey")
        self.textEditKeyPair = QtGui.QLineEdit(Dialog)
        self.textEditKeyPair.setGeometry(QtCore.QRect(120, 140, 411, 31))
        self.textEditKeyPair.setObjectName("textEditKeyPair")
        self.textEditIdentityFile = QtGui.QLineEdit(Dialog)
        self.textEditIdentityFile.setGeometry(QtCore.QRect(120, 180, 321, 31))
        self.textEditIdentityFile.setObjectName("textEditIdentityFile")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "euca2ools configuration", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "URL", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Dialog", "Access key", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Dialog", "Secret key", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("Dialog", "Key pair", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("Dialog", "Identity file", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonBrowse.setText(QtGui.QApplication.translate("Dialog", "Browse", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




import pypatterns.command as CommandModule


import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule
import pypatterns.relational.commands as RelationalCommandModule



class ProvisionVmCommand(CommandModule.Command):


    def __init__(self, automaton, 
                 clusterName, keyPair, imageId):
        CommandModule.Command.__init__(self)
        self._automaton = automaton
        self._clusterName = clusterName
        self._keyPair = keyPair
        self._imageId = imageId
        return


    def execute(self):
        vmInstance = self._automaton.provisionVM(
            self._clusterName, self._keyPair, self._imageId)
        self._vmInstance = vmInstance
        return

    def unexecute(self):

        # TODO:
        # if we are going to call terminateVM
        # then it should not run on a thread
        # self._automaton._terminateVmAndThread()
        self._automaton.terminateVmInstance(self._vmInstance)

        return

    # END class ProvisionVmCommand
    pass


class TerminateVmCommand(CommandModule.Command):

    def __init__(self, automaton, vmInstance):
        CommandModule.Command.__init__(self)
        self._vmInstance = vmInstance
        self._automaton = automaton
        return

    def execute(self):
        self._automaton.terminateVmInstance(self._vmInstance)
        return

    def unexecute(self):
        # what can we do to ensure that there's still a VM?
        return

    # END class TerminateVmCommand
    pass



class UnassignWorkerFromVmCommand(CommandModule.Command):

    def __init__(self, automaton, threadPoolId, vmInstance):
        CommandModule.Command.__init__(self)
        self._automaton = automaton
        self._threadPoolId = threadPoolId
        self._vmInstance = vmInstance
        return

    def execute(self):

        # find the worker thread that's associated with the vm instance
        # decouple the vm and the worker thread
        # dismiss the worker thread
        self._automaton.unassignThreadFromVmInstance(
            self._threadPoolId, self._vmInstance)

        return


    def unexecute(self):

        # for now, undoing this command will do nothing
        # there's a question of how the undo of the SetColumnValueCommand
        # can get the value of the new thread

        # create a new worker thread
        # assign the thread to the vm instance
        #worker = self._automaton.assignThreadToInstance(self._vmInstance)
        #self._worker = worker
        return

    # END class UnasignWorkerFromVmCommand
    pass


class AssignExecuteEnvironmentForVmCommand(CommandModule.Command):

    def __init__(self, automaton, serviceAPI, vmInstance):
        CommandModule.Command.__init__(self)
        self._automaton = automaton
        self._serviceAPI = serviceAPI
        self._vmInstance = vmInstance

        return

    def execute(self):
        env = self._automaton.assignExecuteEnvironmentForVmInstance(
            self._serviceAPI,
            self._vmInstance)
        self._executeEnvironment = env
        pass

    def unexecute(self):
        self._automaton.unassignExecuteEnvironmentFromVmInstance(
            self._vmInstance, executeEnvironment = self._executeEnvironment)
        return

    # END class AssignExecuteEnvironmentForVmCommand
    pass

class AssignThreadForExecuteEnvironmentCommand(CommandModule.Command):

    def __init__(self, automaton, threadPoolId, executeEnvironment):
        CommandModule.Command.__init__(self)
        self._automaton = automaton
        self._threadPoolId = threadPoolId
        self._executeEnvironment = executeEnvironment
        return


    def execute(self):
        worker = self._automaton.assignThreadForExecuteEnvironment(
            self._threadPoolId, self._executeEnvironment)
        self._worker = worker
        return

    def unexecute(self):
        # self._automaton.dismissWorker(worker=self._worker)
        threadPool = self._automaton.getThreadPoolUsingId(self._threadPoolId)
        #self._automaton.unassignThreadFromExecuteEnvironment(
        #    self._threadPoolId, self._executeEnvironment)
        threadPool.dismissWorker(worker=self._worker)
        return

    # END class AssignThreadToExecuteEnvironmentCommand
    pass


class AssignWorkerToVmCommandBuilder(CommandModule.CommandBuilder):

    def __init__(self, command):
        CommandModule.CommandBuilder.__init__(self, command)
        return

    def addCommandsPostExecute(self, commands):
        commands.append(
            AssignWorkerToVmCommand(
                self._command._automaton, 
                self._command._vmInstance
            )
        )
        return

    # END class AssignWorkerToVmCommandBuilder
    pass





class AddNewEntryToVmTableCommandBuilder(CommandModule.CommandBuilder):

    def __init__(self, command):
        CommandModule.CommandBuilder.__init__(self, command)
        return

    def addCommandsPostExecute(self, commands):

        vmInstance = self._command._vmInstance
        automaton = self._command._automaton

        vmTable = automaton.virtualMachineTable()

        command = CommandModule.CompositeCommand()

        addRowCommand = RelationalCommandModule.AddRowCommand(vmTable)
        command.addCommand(addRowCommand)

        for column, value in [('Service', vmInstance.region.name),
                              ('Instance ID', vmInstance.id),
                              ('Image ID', vmInstance.image_id),
                              ('IP', vmInstance.public_dns_name),
                              ('Enabled', 'True')]:

            commandBuilder = \
                           RelationalCommandModule.SetColumnValueCommandBuilder(
                               addRowCommand, column, value
                           )
            command.addCommandBuilder(addRowCommand, commandBuilder)
            pass

        commands.append(command)
        return

    # END class AddNewEntryToVmTableCommandBuilder
    pass






import os
import unittest

import simplejson as ConfigModule

import pomsets.builder as BuilderModule

import pomsets_app.controller.context as ContextModule

import pomsets_app.gui.graph as GraphModule
import pomsets_app.gui.policy as PolicyModule


class TestBasicLayoutPolicy(unittest.TestCase):

    def setUp(self):
        self.builder = BuilderModule.Builder()
        self.policy = PolicyModule.BasicLayoutPolicy()
        return

    def tearDown(self):
        return

    def testLayout1(self):
        builder = self.builder

        pomsetContext = builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        definitionContext = builder.createNewAtomicPomset()
        definitionToReference = definitionContext.pomset()

        nodes = []
        nodes.append(builder.createNewNode(
                pomset, name='A', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='B', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='C', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))

        guiNodeMap = dict([(x, GraphModule.Node(x)) for x in nodes])

        edges = []
        for sourceNode, targetNode in [(nodes[0], nodes[1]),
                                       (nodes[0], nodes[2]),
                                       (nodes[1], nodes[3]),
                                       (nodes[2], nodes[3])]:
            edges.append(
                GraphModule.Edge(
                    builder.connect(
                        pomset,
                        sourceNode, 'temporal output',
                        targetNode, 'temporal input'),
                    [guiNodeMap[sourceNode], None,
                     None, guiNodeMap[targetNode]]
                    )
                )
            pass
        
        self.policy.layoutNodes(guiNodeMap.values(), edges)

        guiNode = guiNodeMap[nodes[0]]
        self.assertEquals((100,300), guiNode.getPosition())

        guiNode = guiNodeMap[nodes[1]]
        self.assertEquals((100,200), guiNode.getPosition())

        guiNode = guiNodeMap[nodes[2]]
        self.assertEquals((250,200), guiNode.getPosition())

        guiNode = guiNodeMap[nodes[3]]
        self.assertEquals((100,100), guiNode.getPosition())

        return


    def testLayout2(self):
        builder = self.builder

        pomsetContext = builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        definitionContext = builder.createNewAtomicPomset()
        definitionToReference = definitionContext.pomset()

        nodes = []
        nodes.append(builder.createNewNode(
                pomset, name='A', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='B', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='C', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))

        guiNodeMap = dict([(x, GraphModule.Node(x)) for x in nodes])

        edges = []
        for sourceNode, targetNode in [(nodes[0], nodes[1]),
                                       (nodes[0], nodes[2]),
                                       (nodes[1], nodes[2]),
                                       (nodes[2], nodes[3]),
                                       (nodes[2], nodes[4])]:
            edges.append(
                GraphModule.Edge(
                    builder.connect(
                        pomset,
                        sourceNode, 'temporal output',
                        targetNode, 'temporal input'),
                    [guiNodeMap[sourceNode], None,
                     None, guiNodeMap[targetNode]]
                    )
                )
            pass
        
        self.policy.layoutNodes(guiNodeMap.values(), edges)

        guiNode = guiNodeMap[nodes[0]]
        self.assertEquals((100,400), guiNode.getPosition())

        guiNode = guiNodeMap[nodes[1]]
        self.assertEquals((100,300), guiNode.getPosition())

        guiNode = guiNodeMap[nodes[2]]
        self.assertEquals((100,200), guiNode.getPosition())

        positions = set([(250,100),(100,100)])

        guiNode = guiNodeMap[nodes[3]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[4]]
        positions.remove(guiNode.getPosition())

        self.assertEquals(0, len(positions))

        return


    def testLayout3(self):
        builder = self.builder

        pomsetContext = builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        definitionContext = builder.createNewAtomicPomset()
        definitionToReference = definitionContext.pomset()

        nodes = []
        nodes.append(builder.createNewNode(
                pomset, name='A', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='B', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='C', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='E', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='F', definitionToReference=definitionToReference))

        guiNodeMap = dict([(x, GraphModule.Node(x)) for x in nodes])

        edges = []
        for sourceNode, targetNode in [(nodes[0], nodes[3]),
                                       (nodes[1], nodes[3]),
                                       (nodes[2], nodes[3]),
                                       (nodes[2], nodes[4]),
                                       (nodes[3], nodes[5])]:
            edges.append(
                GraphModule.Edge(
                    builder.connect(
                        pomset,
                        sourceNode, 'temporal output',
                        targetNode, 'temporal input'),
                    [guiNodeMap[sourceNode], None,
                     None, guiNodeMap[targetNode]]
                    )
                )
            pass
        

        self.policy.layoutNodes(guiNodeMap.values(), edges)

        positions = set([(400,300),(250,300),(100,300)])

        guiNode = guiNodeMap[nodes[0]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[1]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[2]]
        positions.remove(guiNode.getPosition())

        self.assertEquals(0, len(positions))


        positions = set([(250,200),(100,200)])

        guiNode = guiNodeMap[nodes[3]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[4]]
        positions.remove(guiNode.getPosition())

        self.assertEquals(0, len(positions))

        guiNode = guiNodeMap[nodes[5]]
        self.assertEquals((100,100), guiNode.getPosition())

        return


    def testLayout4(self):
        builder = self.builder

        pomsetContext = builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        definitionContext = builder.createNewAtomicPomset()
        definitionToReference = definitionContext.pomset()

        nodes = []
        nodes.append(builder.createNewNode(
                pomset, name='A', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='B', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='C', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))

        guiNodeMap = dict([(x, GraphModule.Node(x)) for x in nodes])

        edges = []
        for sourceNode, targetNode in [(nodes[0], nodes[2]),
                                       (nodes[1], nodes[2]),
                                       (nodes[1], nodes[3])]:
            edges.append(
                GraphModule.Edge(
                    builder.connect(
                        pomset,
                        sourceNode, 'temporal output',
                        targetNode, 'temporal input'),
                    [guiNodeMap[sourceNode], None,
                     None, guiNodeMap[targetNode]]
                    )
                )
            pass
        
        self.policy.layoutNodes(guiNodeMap.values(), edges)

        positions = set([(250,200),(100,200)])

        guiNode = guiNodeMap[nodes[0]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[1]]
        positions.remove(guiNode.getPosition())

        self.assertEquals(0, len(positions))

        positions = set([(250,100),(100,100)])

        guiNode = guiNodeMap[nodes[2]]
        positions.remove(guiNode.getPosition())

        guiNode = guiNodeMap[nodes[3]]
        positions.remove(guiNode.getPosition())

        self.assertEquals(0, len(positions))

        return


    # END class TestBasicLayoutPolicy
    pass



class TestGraphvizLayoutPolicy(unittest.TestCase):

    def setUp(self):
        self.builder = BuilderModule.Builder()
        self.policy = PolicyModule.GraphvizLayoutPolicy()

        contextManager = ContextModule.QtContextManager()
        self.policy.contextManager(contextManager)

        contextManager.resourcePath(['resources'])

        configFilePath = os.path.sep.join(
            contextManager.resourcePath() + 
            ['testdata','TestLayoutPolicy','TestGraphvizLayoutPolicy','config'])


        with open(configFilePath) as f:

            config = ConfigModule.load(f)

            dotPath = config['path for dot']
            assert len(dotPath), 'expected to find path for dot'

            contextManager.commandPath('dot', dotPath)

            return

        raise NotImplemented(
            'could not read credentials from config file %s' % configFilePath)

        return

    def tearDown(self):
        return

    def testLayout1(self):
        builder = self.builder

        pomsetContext = builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        definitionContext = builder.createNewAtomicPomset()
        definitionToReference = definitionContext.pomset()

        nodes = []
        nodes.append(builder.createNewNode(
                pomset, name='A', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='B', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='C', definitionToReference=definitionToReference))
        nodes.append(builder.createNewNode(
                pomset, name='D', definitionToReference=definitionToReference))

        guiNodeMap = dict([(x, GraphModule.Node(x)) for x in nodes])

        edges = []
        for sourceNode, targetNode in [(nodes[0], nodes[1]),
                                       (nodes[0], nodes[2]),
                                       (nodes[1], nodes[3]),
                                       (nodes[2], nodes[3])]:
            edges.append(
                GraphModule.Edge(
                    builder.connect(
                        pomset,
                        sourceNode, 'temporal output',
                        targetNode, 'temporal input'),
                    [guiNodeMap[sourceNode], None,
                     None, guiNodeMap[targetNode]]
                    )
                )
            pass
        
        self.policy.layoutNodes(guiNodeMap.values(), edges)
        return

    # END class GraphvizLayoutPolicy
    pass



from PyQt4.QtCore import *
from PyQt4 import QtGui

import pypatterns.filter as FilterModule

import pomsets.parameter as ParameterModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def definition(self, value=None):
        if value is not None:
            self._definition = value
        if not hasattr(self, '_definition'):
            self._definition = None
        return self._definition

    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)

        return

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        definition = self.definition()

        widget.textEditName.setText(definition.name() or '')
        widget.textEditComment.setText(definition.comment() or '')


        widget.checkBoxCritical.setChecked(definition.isCritical())

        QObject.connect(contextManager.mainWindow(),
                        SIGNAL("OnPomsetModified(PyQt_PyObject)"),
                        self.populateParameters)
        self.populateParameters()

        return


    def populateParameters(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()

        tableParameters = widget.tableParameters
        while tableParameters.rowCount():
            tableParameters.removeRow(0)

        tableParameters.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        tableParameters.setColumnWidth(0, 125)
        tableParameters.setColumnWidth(1, 75)
        tableParameters.setColumnWidth(2, 75)
        tableParameters.setColumnWidth(3, 175)

        filter = FilterModule.ObjectKeyMatchesFilter(
            filter=FilterModule.EquivalenceFilter(ParameterModule.PORT_TYPE_DATA),
            keyFunction = lambda x: x.portType()
            )
        parameters = definition.getParametersByFilter(filter)
        
        tableParameters.setRowCount(len(parameters))

        for rowIndex, parameter in enumerate(parameters):
            name = parameter.id() or ''
            item = QtGui.QTableWidgetItem('name')
            item.setData(Qt.UserRole, parameter)
            item.setData(Qt.DisplayRole, name)
            tableParameters.setItem(rowIndex, 0, item)

            stagingRequired = definition.parameterStagingRequired(name)
            item = QtGui.QTableWidgetItem('staging')
            item.setData(Qt.DisplayRole, stagingRequired)
            item.setData(Qt.UserRole, parameter)
            tableParameters.setItem(rowIndex, 1, item)

            isParameterSweep = definition.isParameterSweep(name)
            item = QtGui.QTableWidgetItem('sweep')
            item.setData(Qt.DisplayRole, isParameterSweep)
            item.setData(Qt.UserRole, parameter)
            tableParameters.setItem(rowIndex, 2, item)

            group = definition.getGroupForParameterSweep(name)
            item = QtGui.QTableWidgetItem('sweep group')
            item.setData(Qt.DisplayRole, str(group))
            item.setData(Qt.UserRole, parameter)
            tableParameters.setItem(rowIndex, 3, item)
            
            pass

        return


    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.ApplyRole:
            self.saveValues()
            pass
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            self.saveValues()
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return


    @pyqtSlot(QModelIndex)
    def on_tableParameters_doubleClicked(self, index):

        parameter = index.data(Qt.UserRole).toPyObject()
        self.editParameterBinding(parameter)

        return

    def editParameterBinding(self, parameter):
        contextManager = self.contextManager()

        # import the modules
        import pomsets_app.gui.qt.edit_parameter_binding.widget as WidgetModule
        import pomsets_app.gui.qt.edit_parameter_binding.controller as ControllerModule

        # initialize the widget
        controller = ControllerModule.Controller(self)
        controller.contextManager(contextManager)
        controller.pomsetContext(self.pomsetContext())
        controller.definition(self.definition())
        controller.parameter(parameter)

        ui = WidgetModule.Ui_Dialog()
        ui.setupUi(controller)

        controller.widget(ui)

        # populate the dialog
        controller.populate()

        controller.show()
        return controller


    @pyqtSlot()
    def on_buttonToggleStaging_clicked(self):

        widget = self.widget()
        tableParameters = widget.tableParameters

        selectedItems = tableParameters.selectedItems()
        if len(selectedItems) is 0:
            return

        # selectedItem = selectedItems[0]
        parameterIds = set([])
        for selectedItem in selectedItems:
            parameter = selectedItem.data(Qt.UserRole).toPyObject()
            parameterId = parameter.id()
            parameterIds.add(parameterId)

        for parameterId in parameterIds:
            stagingRequired = not self.definition().parameterStagingRequired(parameterId)
            self.definition().parameterStagingRequired(
                parameterId, stagingRequired)
            pass

        self.populateParameters()

        return


    @pyqtSlot()
    def on_buttonChangeGroup_clicked(self):

        widget = self.widget()
        tableParameters = widget.tableParameters

        selectedItems = tableParameters.selectedItems()
        if len(selectedItems) is 0:
            return

        selectedItem = selectedItems[0]
        parameter = selectedItem.data(Qt.UserRole).toPyObject()
        self.editParameterSweepGroup(parameter)

        return


    def editParameterSweepGroup(self, parameter):

        contextManager = self.contextManager()

        # import the modules
        import pomsets_app.gui.qt.edit_parameter_sweep_group.widget as WidgetModule
        import pomsets_app.gui.qt.edit_parameter_sweep_group.controller as ControllerModule

        # initialize the widget
        controller = ControllerModule.Controller(self)
        controller.contextManager(contextManager)
        controller.pomsetContext(self.pomsetContext())
        controller.definition(self.definition())
        controller.parameter(parameter)

        ui = WidgetModule.Ui_Dialog()
        ui.setupUi(controller)

        controller.widget(ui)

        # populate the dialog
        controller.populate()

        controller.show()
        return controller


    def saveValues(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        definition = self.definition()
        
        hasBeenModified = False
        
        name = str(widget.textEditName.text())
        if not name == definition.name():
            definition.name(name)
            hasBeenModified = True

        comment = str(widget.textEditComment.text())
        if not comment == definition.comment():
            definition.comment(comment)
            hasBeenModified = True

        isCritical = widget.checkBoxCritical.isChecked()
        if not definition.isCritical() == isCritical:
            definition.isCritical(isCritical)
            hasBeenModified = True

        if hasBeenModified:
            pomsetContext.isModified(True)
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)
            pass

        return


    # END class Controller
    pass



import logging
import os
import sys
import time

import wx

import currypy

import zgl_graphdrawer.Menu as MenuModule
import zgl_graphdrawer.Node as NodeModule
import zgl_graphdrawer.Port as PortModule

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.context as ContextModule
import pomsets.library as LibraryModule

import pomsets_app.gui.event as EventModule
import pomsets_app.utils as UtilsModule

ID_ACCOUNT_MANAGE = wx.NewId()
ID_ACCOUNTMENU_CONNECT = wx.NewId()
ID_ACCOUNTMENU_DISCONNECT = wx.NewId()
ID_ACCOUNT_CONNECT = wx.NewId()
ID_ACCOUNT_DISCONNECT = wx.NewId()

ID_HADOOP_MANAGE = wx.NewId()

ID_EUCA2OOLS_MANAGE = wx.NewId()

ID_DEFINITIONS_EDIT = wx.NewId()

ID_NODE_ADD = wx.NewId()
ID_NODE_REMOVE = wx.NewId()



class NodeContextualMenu(NodeModule.SelectionContextualMenu):


    def bindEvents(self):

        # should be enabled only if the node has not yet been started
        contextManager = self.canvas.contextManager()
        frame = contextManager.app().GetTopWindow()
        pomsetContext, pomset = frame._currentPomsetInfo
        if not contextManager.hasTaskForPomset(pomsetContext.pomset()):
 
            self.addMenuItem('Edit task', self.OnEditTask)

            # should also add "Edit definition"
            # if the definition is not a library definition
            dataNode = self.objects[0].nodeData
            if not dataNode.definitionToReference().isLibraryDefinition():
                self.addMenuItem('Edit definition', self.OnEditTaskDefinition)

            self.addMenuItem('Delete', self.OnDelete)
            self.addMenuItem('Duplicate', self.OnDuplicate)
        else:
            # TODO:
            # These should be enabled only if the node has executed
            # or errored out
            if len(self.objects) == 1:
                self.addMenuItem('View output messages', 
                                 self.OnViewOutputMessages)

        return

    def OnViewOutputMessages(self, event):
        """
        Assumes that this only got called for a single GUI node
        Currently this assumption is enforced by the node contextual menu
        which only adds the menu item to call this function
        if the selection consists only of the node
        """

        dataNode = self.objects[0].nodeData

        import pypatterns.filter as FilterModule
        import pypatterns.relational as RelationalModule
        filter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(dataNode)
        )
        contextManager = self.canvas.contextManager()

        # get the active pomset
        activePomset = contextManager.activePomset()

        # get the task for the active pomset
        if not contextManager.hasTaskForPomset(activePomset):
            print "no task for active pomset %s found" % activePomset
            return
        rootTask = contextManager.getTaskForPomset(activePomset)

        # from the active pomset task, 
        # get the task for the data node
        if not rootTask.hasChildTaskForDefinition(dataNode):
            print "no task for pomset node %s found" % dataNode
            return
        task = rootTask.getChildTaskForDefinition(dataNode)




        # create a dialog that will display all the outputs
        #res =  wx.xrc.XmlResource(
        #    os.path.join(os.getenv('APP_ROOT'),
        #                 'app', 'resources', 'xrc',
        #                 'task output messages dialog.xrc')
        #)
        resourcePath = contextManager.resourcePath()
        res =  wx.xrc.XmlResource(
            os.path.join(resourcePath, 'xrc', 'task output messages dialog.xrc')
            )

        import pomsets_app.gui.widget.task as WidgetModule
        dialog = WidgetModule.MessageDialog(
            res, self.canvas.GetParent()
        )

        # self.event is the event originating from the contextual menu
        dialog.event = self.event
        dialog.contextManager = self.canvas.app().contextManager()
        dialog.populateTaskTree(task)
        dialog._selectedTask = task
        dialog.updateMessageBox()

        try:
            dialog.ShowModal()
        finally:
            dialog.Destroy()

        return


    def OnDuplicate(self, event):
        print "should duplicate node"

        contextManager = self.canvas.contextManager()

        frame = self.canvas.app().GetTopWindow()
        pomsetContext, pomset = frame._currentPomsetInfo
        for guiNode in self.objects:
            dataNode = guiNode.nodeData
            copyDataNode = contextManager.pomsetBuilder().copyNode(
                pomset, dataNode)
            copyDataNode.name('copy of ' + dataNode.name())

            # TODO:
            # this should be relative to the gui node
            # rather than hardcoded to 0, 0
            guiOptions = dataNode.guiOptions()
            if 'canvas position' in guiOptions:
                copyGuiOptions = copyDataNode.guiOptions()
                uiNodeSize = guiNode.getBounds()
                x, y = guiOptions['canvas position']
                x = x + uiNodeSize[0]/2
                y = y + uiNodeSize[1]/2
                copyGuiOptions['canvas position'] = (x,y)
            
            guiEvent = EventModule.NodeCreatedEvent(
                object=copyDataNode,
                contextManager=contextManager)
            contextManager.postEvent(guiEvent)
            pass



        return


    def OnDelete(self, event):

        print "in OnDelete"

        if len(self.objects) is 0:
            return

        frame = self.canvas.app().GetTopWindow()
        pomsetContext, pomset = frame._currentPomsetInfo

        contextManager = self.canvas.contextManager()
        for guiNode in self.objects:
            dataNode = guiNode.nodeData
            contextManager.pomsetBuilder().removeNode(
                pomset, dataNode)

        frame.displayPomset(pomsetContext, pomset)
                
        return


    def OnEditTaskDefinition(self, event):
        import pomsets_app.gui.utils as GuiUtilsModule

        if len(self.objects) is 0:
            return

        guiNode = self.objects[0]
        dataNode =guiNode.nodeData
        definitionToEdit = dataNode.definitionToReference()

        GuiUtilsModule.editDefinition(
            self.canvas.app().contextManager(),
            self.canvas.GetParent(), definitionToEdit)
        return


    def OnEditTask(self, event):

        import pomsets_app.gui.utils as GuiUtilsModule

        if len(self.objects) is 0:
            return

        guiNode = self.objects[0]
        dataNode =guiNode.nodeData
        GuiUtilsModule.editReferenceDefinition(
            self.canvas.app().contextManager(),
            self.canvas.GetParent(), dataNode)
            
        return


    # END class NodeContextualMenu
    pass


class CanvasContextualMenu(MenuModule.CanvasContextualMenu):

    def bindEvents(self):

        self.addMenuItem('Auto arrange', self.OnAutoArrange)

        return

    def OnAutoArrange(self, event):

        self.canvas.computeLayout()
        
        return

    # END class CanvasContextualMenu
    pass


class CreateNodeCanvasContextualMenu(CanvasContextualMenu):
    
    def bindEvents(self):
        CanvasContextualMenu.bindEvents(self)

        # TODO:
        # add a separator here

        self.addMenuItem('Create node', self.OnCreateNode)
        # self.addMenuItem('Create nest', self.OnCreateNest)
        # self.addMenuItem('Create loop', self.OnCreateLoop)
        # self.addMenuItem('Create branch', self.OnCreateBranch)
        return

    def OnCreateNode(self, event):

        import pomsets_app.gui.utils as GuiUtilsModule

        GuiUtilsModule.selectDefinitionAndAddNode(
            self.canvas.app().contextManager(),
            self.canvas.app().GetTopWindow(), event)

        return

    # END class CreateNodeCanvasContextualMenu
    pass


class PortContextualMenu(PortModule.SelectionContextualMenu):


    def bindEvents(self):

        # TODO:
        # only allow editing if pomset execution has not started
        self.addMenuItem('Edit value', self.OnEditValue)

        # TODO:
        # this should enabled, but only for files
        # if the viewer is pre-defined, then just open
        # otherwise, need to query user for the viewer
        # possible to use the OS's filew viewer items?
        # self.addMenuItem('View output')

        return

    
    def OnEditValue(self, event):
        
        if len(self.objects) is 0:
            return
        
        import pomsets_app.gui.utils as GuiUtilsModule
        
        port = self.objects[0]
        uiNode = port.node
        dataNode = uiNode.nodeData
        
        # userParameterName is
        # the name of the parameter that the user right clicked to edit
        # it will be displayed to the user in the dialog box
        # it is also used to retrieve the actual parameter 
        # that should be edited
        userParameterName = port.name

        GuiUtilsModule.editReferenceDefinition(
            self.canvas.app().contextManager(),
            self.canvas.GetParent(), dataNode, userParameterName)
        return
    


    # END PortContextualMenu
    pass


class ActivePomsetContextualMenu(wx.Menu, MenuModule.Menu):

    def __init__(self, frame, pomsetData, *args, **kwds):
        wx.Menu.__init__(self)
    
        MenuModule.Menu.__init__(self, *args, **kwds)
    
        self._frame = frame

        if pomsetData is None:
            pomsetData = (None, None)

        self._selectedPomsetContext = pomsetData[0]
        self._selectedPomset = pomsetData[1]

        return

    def bindEvents(self):
        self.addMenuItem('New', self.OnCreate)

        self.addMenuItem('Load', self.OnLoad)

        if self._selectedPomsetContext is not None:
            
            # need a "save" and "save as"
            if self._selectedPomsetContext.url() is not None:
                self.addMenuItem('Save', self.OnSave)

            self.addMenuItem('Save as', self.OnSaveAs)

            self.addMenuItem('Close', self.OnClose)

            pass

        return

    def OnCreate(self, event):

        frame = self._frame
        contextManager = frame.app().contextManager()

        pomset = contextManager.createNewPomset()


        guiEvent = EventModule.PomsetCreatedEvent(
            object=pomset,
            originalEvent=event,
            contextManager=contextManager)
        contextManager.postEvent(guiEvent)

        return


    def OnLoad(self, event):

        # TODO:
        # this code is the same as MenuEventHandler.OnLoadExistingPomset
        # should consolidate

        import pomset_app.gui.utils as GuiModule
        try:
            window = self._frame
            contextManager = window.app().contextManager()

            path = GuiModule.selectPathToLoadPomsetDefinition(
                contextManager, window, 
                title="Select a pomset to load" )

            if path is not None:
                pomsetContext = ContextModule.loadPomset(path=path)

                guiEvent = EventModule.PomsetLoadedEvent(
                    object=pomsetContext,
                    originalEvent=event,
                    path=path,
                    contextManager=contextManager)

                contextManager.postEvent(guiEvent)
        except Exception, e:
            raise

        return


    def OnSave(self, event):
        window = self._frame
        contextManager = window.app().contextManager()

        ContextModule.savePomset(self._selectedPomsetContext)

        guiEvent = EventModule.PomsetSavedEvent(
            object=self._selectedPomsetContext,
            contextManager = contextManager,
            originalEvent=self.event)
        contextManager.postEvent(guiEvent)

        return


    def OnSaveAs(self, event):
        import pomsets_app.gui.utils as GuiUtilsModule
        
        window = self._frame
        contextManager = window.app().contextManager()
        path = GuiUtilsModule.selectPathToSavePomsetDefinition(
            contextManager, window
            )

        if path is not None:
            ContextModule.savePomsetAs(
                self._selectedPomsetContext,
                path)

            guiEvent = EventModule.PomsetSavedEvent(
                object=self._selectedPomsetContext,
                contextManager = contextManager,
                originalEvent=self.event)
            contextManager.postEvent(guiEvent)

        return


    def OnClose(self, event):
        window = self._frame
        contextManager = window.app().contextManager()
        contextManager.closePomset(self._selectedPomsetContext)
        return


    # END class ActivePomsetContextualMenu
    pass


class LibraryDefinitionContextualMenu(wx.Menu, MenuModule.Menu):
    def __init__(self, frame, selectedContext, *args, **kwds):
        wx.Menu.__init__(self)
    
        MenuModule.Menu.__init__(self, *args, **kwds)
    
        self._frame = frame
        self._selectedContext = selectedContext
        return

    def bindEvents(self):
        if self._selectedContext is not None:
            # add the "Delete menu item
            self.addMenuItem('Remove', self.OnRemoveDefinition)

        return

    def OnRemoveDefinition(self, event):

        context = self._selectedContext
        definition = context.pomset()

        frame = self._frame
        contextManager = frame.app().contextManager()
        persistentLibrary = contextManager.persistentLibrary()
        
        # backup the existing library
        UtilsModule.backupLibrary(
            persistentLibrary.bootstrapLoaderDefinitionsDir())
        
        definitionFilter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(definition)
        )
        
        # remove from transient library
        persistentLibrary.removeDefinition(definitionFilter)
        
        # remove the definition from the file system
        definitionUrl = os.path.sep.join([
                persistentLibrary.bootstrapLoaderDefinitionsDir(),
                definition.url()])

        if not os.path.exists(definitionUrl):
            logging.error('cannot remove definition from url %s' % definitionUrl)
            return
        else:
            os.unlink(definitionUrl)
            
        # save out the new bootstrapper pomset
        # that will not load the just removed pomset
        persistentLibrary.saveBootstrapLoaderPomset()
        
        # update GUI
        frame.populateLibraryDefinitionTreeCtrl()
        frame.populateLoadedDefinitionsPane()
        
        return

    # END class LibraryDefinitionContextualMenu
    pass

class LoadedDefinitionContextualMenu(wx.Menu, MenuModule.Menu):

    def __init__(self, frame, selectedContext, *args, **kwds):
        wx.Menu.__init__(self)
    
        MenuModule.Menu.__init__(self, *args, **kwds)
    
        self._frame = frame
        self._selectedContext = selectedContext
        return

    def bindEvents(self):
        self.addMenuItem('New', self.OnCreateDefinition)
        if self._selectedContext is not None:
            # add the "Edit" menu item
            # add the "Delete menu item
            self.addMenuItem('Edit', self.OnEditDefinition)
            self.addMenuItem('Remove', self.OnRemoveDefinition)

        if self._selectedContext is not None:
            self.AppendSeparator()
            self.addMenuItem('Add to library', self.OnAddDefinitionToLibrary)
            
            # TODO:
            # need a "save" and "save as"
            self.addMenuItem('Save to file', self.OnSaveToFile)
            pass

        self.AppendSeparator()
        self.addMenuItem('Load from file', self.OnLoadFromFile)

        return

    def OnLoadFromFile(self, event):
        frame = self._frame
        contextManager = frame.app().contextManager()
        transientLibrary = contextManager.transientLibrary()

        import pomsets_app.gui.utils as GuiUtilsModule
        try:
            path = GuiUtilsModule.selectPathToLoadPomsetDefinition(
                contextManager, frame,
                title="Select a pomset to load")

            pomsetContext = contextManager.loadExistingPomset(path=path)

            transientLibrary.addPomsetContext(pomsetContext)
            frame.populateLoadedDefinitionsPane()

        except Exception, e:
            logging.error(e)
            raise

        return


    def OnSaveToFile(self, event):
        print "should save definition to file"
        import pomsets_app.gui.utils as GuiUtilsModule
        
        app = self.getApp()
        contextManager = app.contextManager()
        windo = app.GetTopWindow()
        GuiUtilsModule.selectPathToSavePomsetDefinition(
            contextManager, window
            )

        return


    def OnAddDefinitionToLibrary(self, event):

        context = self._selectedContext
        definition = context.pomset()
        frame = self._frame
        contextManager = frame.app().contextManager()
        persistentLibrary = contextManager.persistentLibrary()
        transientLibrary = contextManager.transientLibrary()
        
        # backup the existing library
        UtilsModule.backupLibrary(
            persistentLibrary.bootstrapLoaderDefinitionsDir())
        
        definitionFilter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(definition)
        )
        
        # remove from transient library
        transientLibrary.removeDefinition(definitionFilter)
        
        # define the url for the definition
        definitionPath = definition.id() + '.pomset'
        definition.url(definitionPath)
        outputDir = persistentLibrary.bootstrapLoaderDefinitionsDir()
        
        # save out the definition
        ContextModule.pickleDefinition(
            os.path.join(outputDir, definitionPath), definition)
        
        # add to persistent library
        definition.isLibraryDefinition(True)
        persistentLibrary.addPomsetContext(context)
        
        # save out the bootstrap loader pomset
        # that will load this definition on the next startup
        persistentLibrary.saveBootstrapLoaderPomset()
        
        # update GUI
        frame.populateLibraryDefinitionTreeCtrl()
        frame.populateLoadedDefinitionsPane()
        
        return

    def OnCreateDefinition(self, event):
        import pomsets_app.gui.utils as GuiUtilsModule
    
        context = GuiUtilsModule.createDefinition(
            self._frame.app().contextManager(), self)
    
        self._frame.populateLoadedDefinitionsPane()
        return

    
    def OnEditDefinition(self, event):
        import pomsets_app.gui.utils as GuiUtilsModule
        try:
            # need to pull the context
            context = self._selectedContext
            GuiUtilsModule.editDefinition(
                self._frame.app().contextManager(), 
                self._frame, context.pomset())
            # need to repopulate
        except ValueError, e:
            raise
        return


    def OnRemoveDefinition(self, event):

        context = self._selectedContext
        definition = context.pomset()

        frame = self._frame
        contextManager = frame.app().contextManager()
        transientLibrary = contextManager.transientLibrary()

        # remove from transient library
        definitionFilter = RelationalModule.ColumnValueFilter(
            'definition',
            FilterModule.IdentityFilter(definition)
        )
        transientLibrary.removeDefinition(definitionFilter)

        frame.populateLoadedDefinitionsPane()
        return

    # END class CanvasContextualMenu
    pass



import currypy
import sys
import unittest


import pypatterns.filter as FilterModule
import pomsets.definition as DefinitionModule
import pomsets.parameter as ParameterModule



# TODO:
# this is copied from TestWxApplication
# in the pomsets source code
# need to consolicate
def createShellDefinition(executable, inputParameters, parameterOrderings):
    definition = DefinitionModule.createShellProcessDefinition(
        inputParameters=inputParameters,
        parameterOrderings=parameterOrderings
    )
    definition.executable = currypy.Curry(FilterModule.FUNCTION_IDENTITY,
                                              object=executable)
    return definition


# TODO:
# this is copied from TestWxApplication
# in the pomsets source code
# need to consolicate
def createWordCountDefinition():
    
    parameterOrdering = DefinitionModule.createParameterOrderingTable()
    row = parameterOrdering.addRow()
    row.setColumn('source', 'input file')
    row.setColumn('target', 'output file')
    
    definition = createShellDefinition(
        ['/Users/mjpan/pomsets/trunk/src/test/scripts/wordcount.py'],
        {
            'input file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
            },
            'output file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:True,
            }
        },
        parameterOrdering
    )
    return definition

DEFINITION_WORDCOUNT = createWordCountDefinition()

# TODO:
# this is copied from TestWxApplication
# in the pomsets source code
# need to consolicate
def createWordCountReduceDefinition():
    
    parameterOrdering = DefinitionModule.createParameterOrderingTable()
    row = parameterOrdering.addRow()
    row.setColumn('source', 'input files')
    row.setColumn('target', 'output file')
    
    definition = createShellDefinition(
        ['/Users/mjpan/pomsets/trunk/src/test/scripts/wordcount_reduce.py'],
        {
            'input files':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:{
                    ParameterModule.COMMANDLINE_PREFIX_FLAG:['-input']
                }
            },
            'output file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:{
                    ParameterModule.COMMANDLINE_PREFIX_FLAG:['-output']
                },
                ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:True,
            }
        },
        parameterOrdering
    )
    return definition

DEFINITION_WORDCOUNT_REDUCE = createWordCountReduceDefinition()






from PyQt4.QtCore import *
from PyQt4 import QtGui

import pomsets.parameter as ParameterModule

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):

    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)
        return


    def parameter(self, value=None):
        if value is not None:
            self._parameter = value
        if not hasattr(self, '_parameter'):
            self._parameter = None
        return self._parameter

    def definition(self, value=None):
        if value is not None:
            self._definition = value
        if not hasattr(self, '_definition'):
            self._definition = None
        return self._definition



    def shouldAllowEdit(self, value=None):
        if value is not None:
            self._shouldAllowEdit = value
        if not hasattr(self, '_shouldAllowEdit'):
            self._shouldAllowEdit = None
        return self._shouldAllowEdit


    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        parameter = self.parameter()

        windowTitle = 'View parameter'
        if self.shouldAllowEdit():
            windowTitle = 'Edit parameter'
        self.setWindowTitle(windowTitle)

        textEditName = widget.textEditName
        textEditName.setText(parameter.id() or '')
        textEditDescription=widget.textEditDescription
        textEditDescription.setText(parameter.description() or '')

        comboBoxDirection = widget.comboBoxDirection
        comboBoxDirection.addItems([
                ParameterModule.PORT_DIRECTION_INPUT,
                ParameterModule.PORT_DIRECTION_OUTPUT])
        direction = parameter.portDirection()
        if parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT):
            direction = ParameterModule.PORT_DIRECTION_OUTPUT
        index = comboBoxDirection.findText(direction)
        if index >= 0:
            comboBoxDirection.setCurrentIndex(index)

        checkBoxCommandline = widget.checkBoxCommandline
        isCommandline = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE)
        checkBoxCommandline.setChecked(isCommandline)

        textEditFlag = widget.textEditFlag
        if isCommandline:

            commandlineOptions = parameter.getAttribute(
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
                defaultValue = {}
            )

            prefixFlag = commandlineOptions.get(
                ParameterModule.COMMANDLINE_PREFIX_FLAG, []
            )
            textEditFlag.setText(' '.join(prefixFlag))
            pass

        
        checkBoxOptional = widget.checkBoxOptional
        isOptional = parameter.optional()
        checkBoxOptional.setChecked(isOptional)

        checkBoxFile = widget.checkBoxFile
        isFile =  parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE) or \
            parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT)
        checkBoxFile.setChecked(isFile)

        checkBoxList = widget.checkBoxList
        argumentValueIsList = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_ISLIST)
        checkBoxList.setChecked(argumentValueIsList)

        if isCommandline and argumentValueIsList:
            checkBoxDistributeFlag = widget.checkBoxDistributeFlag
            shouldDistributePrefixFlag = commandlineOptions.get(
                ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE, False)
            checkBoxDistributeFlag.setChecked(shouldDistributePrefixFlag)
            pass

        self.setVisibilityOfWidgetItems()

        if not self.shouldAllowEdit():
            textEditName.setReadOnly(True)
            textEditDescription.setReadOnly(True)
            comboBoxDirection.setEditable(False)
            comboBoxDirection.setEnabled(False)
            checkBoxCommandline.setEnabled(False)
            textEditFlag.setReadOnly(True)
            checkBoxOptional.setEnabled(False)
            checkBoxFile.setEnabled(False)
            checkBoxList.setEnabled(False)
            widget.checkBoxDistributeFlag.setEnabled(False)

        return


    def setVisibilityOfWidgetItems(self):
        widget = self.widget()

        checkBoxList = widget.checkBoxList
        checkboxCommandline = widget.checkBoxCommandline

        isList = checkBoxList.isChecked()
        isCommandline = checkboxCommandline.isChecked()

        shouldShowFlag = True
        shouldShowDistributeFlag = True
        # enable the flag items only if commandline is checked
        if isCommandline:
            shouldShowFlag = True
            # enable the distribute flag items only if 
            # both commandline and list are checked
            if isList:
                shouldShowDistributeFlag = True
            else:
                shouldShowDistributeFlag = False
            pass
        else:
            shouldShowFlag = False
            shouldShowDistributeFlag = False
            pass

        for childWidget, visibilitySetting in [
            (widget.labelFlag, shouldShowFlag),
            (widget.textEditFlag, shouldShowFlag),
            (widget.checkBoxDistributeFlag, shouldShowDistributeFlag)]:
            childWidget.setVisible(visibilitySetting)

        return


    @pyqtSlot(QtGui.QAbstractButton)
    def on_buttonBox_clicked(self, button):
        widget = self.widget()
        buttonRole = widget.buttonBox.buttonRole(button)
        if buttonRole == QtGui.QDialogButtonBox.ApplyRole:
            if self.shouldAllowEdit():
                self.saveValues()
            pass
        if buttonRole == QtGui.QDialogButtonBox.AcceptRole:
            if self.shouldAllowEdit():
                self.saveValues()
            self.accept()
            pass
        if buttonRole == QtGui.QDialogButtonBox.RejectRole:
            self.reject()
            pass
        return


    @pyqtSlot(int)
    def on_checkBoxList_stateChanged(self, newState):
        self.setVisibilityOfWidgetItems()
        return

    @pyqtSlot(int)
    def on_checkBoxCommandline_stateChanged(self, newState):
        self.setVisibilityOfWidgetItems()
        return


    def saveValues(self):

        widget = self.widget()
        contextManager = self.contextManager()
        pomsetContext = self.pomsetContext()
        parameter = self.parameter()
        
        hasBeenModified = False
        
        name = str(widget.textEditName.text())
        if not name == parameter.id():
            self.definition().renameParameter(parameter.id(), name)
            hasBeenModified = True

        description = str(widget.textEditDescription.text())
        if not description == parameter.description():
            parameter.description(description)
            hasBeenModified = True

        direction = str(widget.comboBoxDirection.currentText())
        isFile = widget.checkBoxFile.isChecked()
        isInputFile = False
        isSideEffect = False
        if isFile:
            if direction == ParameterModule.PORT_DIRECTION_OUTPUT:
                direction = ParameterModule.PORT_DIRECTION_INPUT
                isSideEffect = True
            else:
                isInputFile = True
        
        if not parameter.portDirection() == direction:
            parameter.portDirection(direction)
            hasBeenModified = True
        if not parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT) == isSideEffect:
            parameter.setAttribute(
                ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT, isSideEffect)
            hasBeenModified = True
        if not parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE) == isInputFile:
            parameter.setAttribute(
                ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE, isInputFile)
            hasBeenModified = True

        checkBoxCommandline = widget.checkBoxCommandline
        isCommandline = checkBoxCommandline.isChecked()
        if not parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE) == isCommandline:
            parameter.setAttribute(
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE, isCommandline)
            hasBeenModified = True

        commandlineOptions = {}
        if isCommandline:

            # flag
            textEditFlag = widget.textEditFlag
            prefixFlag = str(textEditFlag.text()).split(' ')
            commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG] = prefixFlag

            # distribute flag
            checkBoxDistributeFlag = widget.checkBoxDistributeFlag
            shouldDistributeFlag = checkBoxDistributeFlag.isChecked()
            commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE] = shouldDistributeFlag
            pass

        currentCommandlineOptions = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
            defaultValue = {}
            )
        if not currentCommandlineOptions == commandlineOptions:
            parameter.setAttribute(
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS,
                commandlineOptions
            )
            hasBeenModified = True


        # handle whether the parameter is optional
        checkBoxOptional = widget.checkBoxOptional
        isOptional = checkBoxOptional.isChecked()
        if not parameter.optional() == isOptional:
            parameter.optional(isOptional)
            hasBeenModified = True


        if hasBeenModified:

            pomsetContext.isModified(True)
            
            contextManager.mainWindow().emit(
                SIGNAL("OnPomsetModified(PyQt_PyObject)"), pomsetContext)
        return


    # END class Controller
    pass



import os

import pomsets_app.controller.automaton as AutomatonModule


class Frame(object):

    def app(self, value=None):
        if value is not None:
            self._app = value
        return self._app

    def displayPomset(self, pomsetContext, pomsetReferencePath=None):
        
        canvas = self.canvas

        contextManager = self.app().contextManager()

        if pomsetContext is None:
            contextManager.currentDisplayedPomsetInfo((None, None, False))
            canvas.breadCrumbs('No active pomset to display')
            canvas.OnRefresh()
            return

        # after this point
        # it is assumed that pomsetContext is not None
        if pomsetReferencePath is None:
            pomsetReferencePath = [pomsetContext.reference()]
        pomsetReference = pomsetContext.reference()

        contextManager.currentDisplayedPomsetInfo(
            (pomsetContext, pomsetReferencePath, True))

        # display the actual pomset in the canvas
        canvas.displayPomset(pomsetReferencePath)

        """
        if contextManager.hasTaskForPomsetPath(pomsetReferencePath):
            # TODO:
            # need to handle the case where
            # we are displaying a deeply nested pomset
            rootTask = contextManager.getTaskForPomsetPath(pomsetReferencePath)
            canvas.updateExecutionStatus(rootTask)
        """

        self.setBreadCrumbs(pomsetContext, pomsetReferencePath)
        
        return


    def updatePomsetMenu(self):
        contextManager = self.app().contextManager()
        activePomsetContext = contextManager.activePomsetContext()
        #self.setMenuItemActive(
        #    'Pomset', 'Close', activePomsetContext is not None)

        return
    
    def updateExecuteMenus(self):

        contextManager = self.app().contextManager()
        pomsetContext, pomsetReferencePath, isEditable = \
            contextManager.currentDisplayedPomsetInfo()

        # update the menus
        shouldEnableExecute = True
        if contextManager.hasTaskForPomsetPath(pomsetReferencePath):
            # disable the execute menu
            shouldEnableExecute = False
            pass

        try:
            threadPoolId = AutomatonModule.ID_EXECUTE_EUCA2OOLS
            threadPool = contextManager.automaton().getThreadPoolUsingId(
                threadPoolId)
            shouldEnableExecuteEuca2ools = shouldEnableExecute
            if threadPool.isEmpty():
                # disable the execute eucalyptus menu item
                shouldEnableExecuteEuca2ools = False
                pass
        except ValueError, e:
            # this is probably because the threadpool
            # for euca2ools was not created
            shouldEnableExecuteEuca2ools = False
            pass

        #self.setMenuItemActive('Execute', 'Local', shouldEnableExecute)
        #self.setMenuItemActive('Execute', 'Euca2ools', shouldEnableExecuteEuca2ools)

        return

    def selectActivePomset(self, pomset):
        
        # currently a placeholder
        # need to figure out the pomset currently being displayed
        # and select that in the GUI
        if self._updateActivePomsetTreeCtrlSelection:
            print "should update selection"

        
        # set the breadcrumbs for the pomset to be displayed
        contextManager = self.app().contextManager()
        activeContext = contextManager.activePomsetContext()

        return


    def setBreadCrumbs(self, pomsetContext, contextPath=None):
        if contextPath is None:
            contextPath = [pomsetContext.reference()]

        canvas = self.canvas
        canvas.breadCrumbs(' > '.join([x.name() for x in contextPath]))
        
        return

    # END class Frame
    pass





from __future__ import with_statement

import os
import sys
import Image, ImageDraw

"""
img = Image.new("RGBA", (300,300), "#FFFFFF")
draw = ImageDraw.Draw(img)
draw.line((0,10,0,300),fill=(0,0,0),width=10)
"""

KEY_BORDER_SELECTION_TRUE =  [0.3, 0.3, 0.3, 1.0]
KEY_BORDER_SELECTION_FALSE = [0.7, 0.7, 0.7, 1.0]
KEY_BACKGROUND_SELECTION_TRUE = [0.6, 0.6, 0.9, 1.0]
#KEY_BACKGROUND_SELECTION_FALSE = [0.3, 0.3, 0.5, 0.8]
KEY_BACKGROUND_SELECTION_FALSE = [0.6, 0.6, 0.6, 1.0]
KEY_TEXT_SELECTION_TRUE = [1.0, 1.0, 1.0, 1.0]
KEY_TEXT_SELECTION_FALSE = [0.8, 0.8, 0.8, 1.0]

BORDER_WIDTH = 10

DEFAULT_NODE_WIDTH = 300
DEFAULT_NODE_HEIGHT = 200


def saveImage(image, path):
    #with open(path, 'w') as f:
    #    image.save(f, 'PNG')
    image.save(path, 'PNG')
    return

def generateNewImage(size=None, color=None):
    if size is None:
        size = (DEFAULT_NODE_WIDTH,DEFAULT_NODE_HEIGHT)
    if color is None:
        # color = "#FFFFFF"
        color = "#0000FF"

    img = Image.new("RGBA", size, color)
    return img

def getFill(color):
    # return tuple(color[:3])
    fill = "#%s%s%s%s%s%s" % (
        hex(int(color[0]*255)/16)[2],
        hex(int(color[0]*255)%16)[2],
        hex(int(color[1]*255)/16)[2],
        hex(int(color[1]*255)%16)[2],
        hex(int(color[2]*255)/16)[2],
        hex(int(color[2]*255)%16)[2],
        )

    return fill


def generateAtomicNode(backgroundColor, borderColor):

    imageWidth = DEFAULT_NODE_WIDTH
    imageHeight = DEFAULT_NODE_HEIGHT

    img = generateNewImage(size=(imageWidth,imageHeight),
                           color=backgroundColor)
    draw = ImageDraw.Draw(img)

    draw.line((0,BORDER_WIDTH/2,imageWidth,BORDER_WIDTH/2), 
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((0,imageHeight-BORDER_WIDTH/2,imageWidth,imageHeight-BORDER_WIDTH/2), 
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((BORDER_WIDTH/2, 0, BORDER_WIDTH/2, imageHeight), 
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-BORDER_WIDTH/2, 0, imageWidth-BORDER_WIDTH/2, imageHeight), 
              fill=borderColor,
              width=BORDER_WIDTH)

    return img


def generateNestNode(backgroundColor,borderColor):
    imageWidth = DEFAULT_NODE_WIDTH
    imageHeight = DEFAULT_NODE_HEIGHT

    offsetWidth = int(1.5 * BORDER_WIDTH)
    offsetHeight = int(1.5 * BORDER_WIDTH)

    offsetBorderWidth = 5
    offsetBorderHeight = 5

    img = generateNewImage(size=(imageWidth,imageHeight),
                           color=backgroundColor)
    draw = ImageDraw.Draw(img)

    draw.line((0,offsetBorderHeight,imageWidth,offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-offsetBorderWidth,0,imageWidth-offsetBorderWidth,
               imageHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth,imageHeight-offsetBorderHeight,
               0,imageHeight-offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((0+offsetBorderWidth,imageHeight, 0+offsetBorderWidth,0),
              fill=borderColor,
              width=BORDER_WIDTH)

    draw.line((offsetWidth,offsetHeight+offsetBorderHeight,imageWidth-offsetWidth,offsetHeight+offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-offsetWidth-offsetBorderWidth,offsetHeight,
               imageWidth-offsetWidth-offsetBorderWidth,imageHeight-offsetHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-offsetWidth,imageHeight-offsetHeight-offsetBorderHeight,
               offsetWidth,imageHeight-offsetHeight-offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((offsetWidth+offsetBorderWidth,imageHeight-offsetHeight,
               offsetWidth+offsetBorderWidth,offsetHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    return img

def generateBranchNode():
    return

def generateLoopNode():
    return

def generateParameterSweepNode(backgroundColor, borderColor):

    offsetWidth = int(1.5 * BORDER_WIDTH)
    offsetHeight = int(1.5 * BORDER_WIDTH)

    offsetBorderWidth = -5
    offsetBorderHeight = 5

    imageWidth = DEFAULT_NODE_WIDTH + 3*BORDER_WIDTH
    imageHeight = DEFAULT_NODE_HEIGHT + 3*BORDER_WIDTH
    img = generateNewImage(size=(imageWidth,imageHeight),
                           color=backgroundColor)

    draw = ImageDraw.Draw(img)

    draw.line((offsetWidth,2*offsetHeight+offsetBorderHeight+3,
               offsetWidth,offsetHeight+offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((offsetWidth+offsetBorderWidth,offsetHeight+offsetBorderHeight+2,
               imageWidth-offsetWidth,offsetHeight+offsetBorderHeight+2),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-offsetWidth+offsetBorderWidth,offsetHeight+offsetBorderHeight,
               imageWidth-offsetWidth+offsetBorderWidth,imageHeight-offsetHeight+offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth-offsetWidth+offsetBorderWidth,imageHeight-offsetHeight,
               imageWidth-2*offsetWidth,imageHeight-offsetHeight),
              fill=borderColor,
              width=BORDER_WIDTH)

    draw.line((offsetWidth*2,offsetHeight+offsetBorderHeight,
               2*offsetWidth,0),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((2*offsetWidth+offsetBorderWidth+1,0+offsetBorderHeight,
               imageWidth,0+offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth+offsetBorderWidth,0+offsetBorderHeight, 
               imageWidth+offsetBorderWidth,imageHeight-2*offsetHeight+offsetBorderHeight),
              fill=borderColor,
              width=BORDER_WIDTH)
    draw.line((imageWidth+offsetBorderWidth,imageHeight-2*offsetHeight,
               imageWidth-offsetWidth,imageHeight-2*offsetHeight),
              fill=borderColor,
              width=BORDER_WIDTH)

    return img


def main(args):

    if len(args) < 2:
        print "need to specify output dir"
        return

    outputDir = args[1]

    for path, background, border in [('node unselected.png',
                                      KEY_BACKGROUND_SELECTION_FALSE,
                                      KEY_BORDER_SELECTION_FALSE),
                                     ('node selected.png',
                                      KEY_BACKGROUND_SELECTION_TRUE,
                                      KEY_BORDER_SELECTION_TRUE)]:
        backgroundColor = getFill(background)
        borderColor = getFill(border)
        img = generateAtomicNode(backgroundColor, borderColor)
        saveImage(img, os.path.join(outputDir, path))
        pass

    for path, background, border in [('nest unselected.png',
                                      KEY_BACKGROUND_SELECTION_FALSE,
                                      KEY_BORDER_SELECTION_FALSE),
                                     ('nest selected.png',
                                      KEY_BACKGROUND_SELECTION_TRUE,
                                      KEY_BORDER_SELECTION_TRUE)]:
        backgroundColor = getFill(background)
        borderColor = getFill(border)
        img = generateNestNode(backgroundColor, borderColor)
        saveImage(img, os.path.join(outputDir, path))
        pass

    generateBranchNode()
    generateLoopNode()


    for path, background, border in [('ps unselected.png',
                                      KEY_BACKGROUND_SELECTION_FALSE,
                                      KEY_BORDER_SELECTION_FALSE),
                                     ('ps selected.png',
                                      KEY_BACKGROUND_SELECTION_TRUE,
                                      KEY_BORDER_SELECTION_TRUE)]:
        backgroundColor = getFill(background)
        borderColor = getFill(border)
        img = generateParameterSweepNode(backgroundColor, borderColor)
        saveImage(img, os.path.join(outputDir, path))
        pass

    return

if __name__=="__main__":
    main(sys.argv)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/cloud pool dialog.ui'
#
# Created: Fri Mar  5 17:45:33 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(881, 421)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(500, 380, 341, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonCredentials = QtGui.QPushButton(Dialog)
        self.buttonCredentials.setGeometry(QtCore.QRect(40, 380, 115, 32))
        self.buttonCredentials.setObjectName("buttonCredentials")
        self.buttonAssign = QtGui.QPushButton(Dialog)
        self.buttonAssign.setGeometry(QtCore.QRect(160, 380, 115, 32))
        self.buttonAssign.setObjectName("buttonAssign")
        self.buttonUnassign = QtGui.QPushButton(Dialog)
        self.buttonUnassign.setGeometry(QtCore.QRect(280, 380, 115, 32))
        self.buttonUnassign.setObjectName("buttonUnassign")
        self.buttonRefresh = QtGui.QPushButton(Dialog)
        self.buttonRefresh.setGeometry(QtCore.QRect(400, 380, 115, 32))
        self.buttonRefresh.setObjectName("buttonRefresh")
        self.tableInstances = QtGui.QTableWidget(Dialog)
        self.tableInstances.setGeometry(QtCore.QRect(20, 20, 841, 351))
        self.tableInstances.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tableInstances.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.tableInstances.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.tableInstances.setObjectName("tableInstances")
        self.tableInstances.setColumnCount(8)
        self.tableInstances.setRowCount(0)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(1, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(2, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(3, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(4, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(5, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(6, item)
        item = QtGui.QTableWidgetItem()
        self.tableInstances.setHorizontalHeaderItem(7, item)
        self.tableInstances.horizontalHeader().setVisible(True)
        self.tableInstances.verticalHeader().setVisible(False)

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Manage cloud pool", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonCredentials.setText(QtGui.QApplication.translate("Dialog", "Credentials", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonAssign.setText(QtGui.QApplication.translate("Dialog", "Assign", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonUnassign.setText(QtGui.QApplication.translate("Dialog", "Unassign", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonRefresh.setText(QtGui.QApplication.translate("Dialog", "Refresh", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(0).setText(QtGui.QApplication.translate("Dialog", "Service", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(1).setText(QtGui.QApplication.translate("Dialog", "API", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(2).setText(QtGui.QApplication.translate("Dialog", "Reservation ID", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(3).setText(QtGui.QApplication.translate("Dialog", "Instance ID", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(4).setText(QtGui.QApplication.translate("Dialog", "Image ID", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(5).setText(QtGui.QApplication.translate("Dialog", "IP Address", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(6).setText(QtGui.QApplication.translate("Dialog", "Key name", None, QtGui.QApplication.UnicodeUTF8))
        self.tableInstances.horizontalHeaderItem(7).setText(QtGui.QApplication.translate("Dialog", "Assigned", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'resources/qt/edit parameter sweep group.ui'
#
# Created: Sun Mar 14 15:53:21 2010
#      by: PyQt4 UI code generator 4.7
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(401, 348)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(190, 300, 191, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonUseOwnGroup = QtGui.QRadioButton(Dialog)
        self.buttonUseOwnGroup.setGeometry(QtCore.QRect(20, 20, 181, 20))
        self.buttonUseOwnGroup.setObjectName("buttonUseOwnGroup")
        self.buttonUseOtherGroup = QtGui.QRadioButton(Dialog)
        self.buttonUseOtherGroup.setGeometry(QtCore.QRect(20, 50, 211, 20))
        self.buttonUseOtherGroup.setObjectName("buttonUseOtherGroup")
        self.listWidget = QtGui.QListWidget(Dialog)
        self.listWidget.setGeometry(QtCore.QRect(20, 90, 361, 192))
        self.listWidget.setObjectName("listWidget")

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Specify parameter sweep group", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonUseOwnGroup.setText(QtGui.QApplication.translate("Dialog", "Use own group", None, QtGui.QApplication.UnicodeUTF8))
        self.buttonUseOtherGroup.setText(QtGui.QApplication.translate("Dialog", "Become member of group", None, QtGui.QApplication.UnicodeUTF8))


if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Dialog = QtGui.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




import logging
import os

import StringIO

from PyQt4.QtCore import *
from PyQt4 import QtGui

import pomsets_app.gui.qt as QtModule

class Controller(QtModule.Controller, QtGui.QDialog):


    def __init__(self, *args, **kwds):
        QtGui.QDialog.__init__(self, *args, **kwds)
        QtModule.Controller.__init__(self)

        return

    def populate(self):

        widget = self.widget()
        contextManager = self.contextManager()

        stream = StringIO.StringIO()
        contextManager.outputCommandsToStream(stream)

        textEditCommands = widget.textEditCommands
        textEditCommands.setText(stream.getvalue())

        return


    # END class Controller
    pass



import math
import os
import subprocess
import tempfile

import pomsets.resource as ResourceModule

class Policy(ResourceModule.Struct):

    ATTRIBUTES = ['contextManager']

    def __init__(self):
        ResourceModule.Struct.__init__(self)
        return

    # END clas Policy
    pass


class NodePolicy(Policy):

    ATTRIBUTES = Policy.ATTRIBUTES + [
        'portPolicy',
        'visualPolicy'
        ]

    WIDTH = 100
    HEIGHT = 50

    KEY_BORDER_SELECTION_TRUE = "selected border"
    KEY_BORDER_SELECTION_FALSE = "not selected border"
    KEY_BACKGROUND_SELECTION_TRUE = "selected background"
    KEY_BACKGROUND_SELECTION_FALSE = "not selected background"
    KEY_TEXT_SELECTION_TRUE = "selected text"
    KEY_TEXT_SELECTION_FALSE = "not selected text"

    def __init__(self):
        Policy.__init__(self)
        self._colors = {}
        return

    def color(self, key, value=None):
        if value is not None:
            self._colors[key] = value
        return self._colors[key]

    def getPositionForNameLabel(self):
        return [0, 15, 0]

    def getPositionForExecuteStatusLabel(self):
        return [0, -7, 0]

    def setupDimensions(self, node):
        node.width= NodePolicy.WIDTH
        node.height= NodePolicy.HEIGHT
        return

    def setPortPosition(self, node, port):

        # TODO:
        # some of the values in this function
        # are calculated multiple times
        # because this function is called once per port
        # but some of the values, 
        # e.g. minimumPortAreaWidth
        # is constant

        import pomsets_app.gui.graph as GraphModule

        portDirection = port.direction

        yOffset = 0
        if (portDirection == GraphModule.Port.PORT_DIRECTION_INPUT):
            yOffset = node.height + 5
        elif (portDirection == GraphModule.Port.PORT_DIRECTION_OUTPUT):
            yOffset = -1*(port.height+5)

        port.y = yOffset

        minimumPortMargin = 5
        portCount = port.positionCount()
        minimumPortAreaWidth = \
            port.width * portCount + minimumPortMargin * (portCount-1)

        portIndex = port.positionIndex()

        xOffset = 0
        if minimumPortAreaWidth > node.width:
            # Ports have to extend beyond the limit of the node
            # because of size constraint
            startLocation = (node.width-minimumPortAreaWidth)/2
            xOffset = startLocation + portIndex*(portWidth + minimumPortMargin) - portWidth/2
        else:
            xOffset = node.width/portCount * (portIndex+0.5) - port.width/2
        port.x = xOffset


        return 


    # END class NodePolicy
    pass


class VisualPolicy(Policy):

    ATTRIBUTES = Policy.ATTRIBUTES + []
    
    KEY_SELECTION_TRUE = "selected"
    KEY_SELECTION_FALSE = "not selected"

    COLOR_DEFAULT = [1.0, 1.0, 1.0, 1.0]

    FONT_DEFAULT = "FreeUniversal-Regular"
    FONT_DEFAULT_FILE = "%s.ttf" % FONT_DEFAULT

    KEY_FONT_DEFAULT = "default font"    

    def __init__(self):
        Policy.__init__(self)
        self._backgroundMap = {}
        self._textMap = {}
        self._colorMap = {}
        self._fontMap = {}
        return

    def getFontPath(self):
        return os.sep.join([self.contextManager().resourcePath(),
                            'fonts'])

    def color(self, key, value=None):
        if value is not None:
            self._colorMap[key] = value
        if not key in self._colorMap:
            self._colorMap[key] = copy.copy(VisualPolicy.COLOR_DEFAULT)
        return self._colorMap[key]


    def getPathForDefaultFont(self):
        fontPath = os.sep.join([self.getFontPath(),
                                VisualPolicy.FONT_DEFAULT_FILE])
        return fontPath
    
    # END class VisualPolicy
    pass



class EdgePolicy(Policy):


    KEY_SELECTION_TRUE = 'selected'
    KEY_SELECTION_FALSE = 'not selected'
    KEY_DYNAMIC = 'dynamic'

    ATTRIBUTES = Policy.ATTRIBUTES + [
        'visualPolicy',
        'lineColor'
        ]

    def __init__(self):
        Policy.__init__(self)
        return

    def updateEdge(self, edge):

        inputPort = edge.inputPort()
        inputPort.edge = edge
        inputPort.updateColors()

        outputPort = edge.outputPort()
        outputPort.edge = edge
        outputPort.updateColors()

        pass


    def createPath(self, inputPoint, outputPoint, steps=30):

        points = []
        delta = (outputPoint[0]-inputPoint[0], outputPoint[1]-inputPoint[1])
        
        for i in xrange(steps+1):
            theta = float(i)/steps
            x = inputPoint[0] + delta[0] * (1-math.cos(theta*math.pi))/2
            y = inputPoint[1] + delta[1] * theta
            z = math.sin(theta*math.pi)*-1.0
            points.append( (x,y,z) )
        return points

    # END class EdgePolicy
    pass


class BasicLayoutPolicy(Policy):

    ATTRIBUTES = Policy.ATTRIBUTES + []

    OFFSET_X = 100
    OFFSET_Y = 100

    def __init__(self):
        Policy.__init__(self)
        return

    def layoutNodes(self, nodes, edges):

        # sort the nodes
        predecessorMap = {}
        successorMap = {}
        for edge in edges:
            sourceNode = edge.sourceNode()
            targetNode = edge.targetNode()
            if not targetNode in predecessorMap:
                predecessorMap[targetNode] = []
                pass
            predecessorMap[targetNode].append(sourceNode)
            if not sourceNode in successorMap:
                successorMap[sourceNode] = []
            successorMap[sourceNode].append(targetNode)
            pass

        minimalNodes = set(nodes).difference(
            set(reduce(list.__add__, successorMap.values(), [])))
        nodeToLevelMap = dict([(x,0) for x in minimalNodes])

        processedNodes = set(minimalNodes)
        nodesToProcess = reduce(
            list.__add__,
            [successorMap[x] for x in minimalNodes if x in successorMap], [])
        while len(nodesToProcess):
            nodeToProcess = nodesToProcess.pop(0)
            processedNodes.add(nodeToProcess)

            # add the node's successors to nodesToProces
            nodesToProcess.extend(successorMap.get(nodeToProcess, []))

            nodePredecessors = predecessorMap.get(nodeToProcess, [])
            unprocessedPredecessors = set(nodePredecessors).difference(processedNodes)
            if len(unprocessedPredecessors) is not 0:
                # we have not processed all its predecessors,
                # so we don't do anything
                continue
            nodeToLevelMap[nodeToProcess] = max(
                [nodeToLevelMap[x] for x in nodePredecessors]) + 1
            pass
        
        # place the nodes into the different levels
        levelToNodeMap = {}
        for node, level in nodeToLevelMap.iteritems():
            if not level in levelToNodeMap:
                levelToNodeMap[level] = []
            levelToNodeMap[level].append(node)

        # now set the position accordingly
        numLevels = len(levelToNodeMap)

        offsetX = NodePolicy.WIDTH + 50
        offsetY = NodePolicy.HEIGHT + 50

        height = offsetY * numLevels
        width =  offsetX * max([len(x) for x in levelToNodeMap.values()])
        for level, nodesInLevel in levelToNodeMap.iteritems():
            y = offsetY * (numLevels - level - 1) + BasicLayoutPolicy.OFFSET_Y 
            for index, nodeInLevel in enumerate(nodesInLevel):
                x = offsetX * index + BasicLayoutPolicy.OFFSET_X
                nodeInLevel.setPosition(x, y)
            pass

        return (width, height)


    # END class LayoutPolicy
    pass


class GraphvizLayoutPolicy(Policy):
    
    ATTRIBUTES = Policy.ATTRIBUTES + []

    def __init__(self):
        Policy.__init__(self)
        return

    def temporaryFilePath(self, value=None):
        if value is not None:
            self._tmpFile = None
            
        if not hasattr(self, '_tmpFile'):
            self._tmpFile = os.sep.join(
                [tempfile.gettempdir(), 'temp_dotFile.dot'])
            
        return self._tmpFile



    def generateDotFile(self, path, nodes, edges):

        import gv
        
        nodeMap = {}

        gvGraph = gv.digraph('name')
        for node in nodes:
            gvNode = gv.node(gvGraph, node.nodeData.name())
            nodeMap[node] = gvNode
            gv.setv(gvNode, 'pos', str(node.getPosition()))

        for edge in edges:
            gvEdge = gv.edge(gvGraph, 
                             edge.sourceNode().nodeData.name(),
                             edge.targetNode().nodeData.name())

        gv.write(gvGraph, path)
        return



    def layoutNodes(self, nodes, edges):

        tmpFile = self.temporaryFilePath()
        self.generateDotFile(tmpFile, nodes, edges)

        # use the subprocess module to execute
        layoutFile = '/tmp/pomset.layout'
        command = [self.contextManager().commandPath('dot'),
                   tmpFile, '-o%s' % layoutFile]

        try:
            ret = subprocess.call(command)
        except OSError:
            pass

        boundingbox = ""

        with open(layoutFile, 'r') as f:
            for line in f.readlines():
                logging.debug(line)

                match = re.search('^\s*graph \[(.*)\];$', line)
                if match is not None:
                    attributeString = match.group(1)
                    logging.debug("Graph has attributes: %s" % match.group(1))
                    attributes = self.getAttributesDict(attributeString)
                    if attributes.has_key('bb'):
                        bb = map(float, attributes['bb'].split(','))
                    continue;

                foundNode = False
                for node in nodes:
                    nodeName = "%s" % (node.nodeData.name())
                    match = re.search('^\s*"{0,1}'+nodeName+'"{0,1}\s*\[(.*)\];$', line)
                    if match is not None:
                        attributeString = match.group(1)
                        attributes = self.getAttributesDict(attributeString)
                        if attributes.has_key('pos'):

                            position = 1.5 * numpy.matrix(
                                ' '.join(attributes['pos'].split(',')))
                            logging.debug('position >> %s' % position)
                            node.setPosition(position[0,0], position[0,1])
                            pass
                        if attributes.has_key('width'):
                            (node.width) = float(attributes['width']) * 80
                        if attributes.has_key('height'):
                            (node.height) = float(attributes['height']) * 80
                        foundNode = True;
                    if foundNode:
                        break

        return (bb[2],bb[3])

    
    def getAttributesDict(self, attributeString):
        dict = {}
        matches = re.findall('(\w+)="(.*?)"',attributeString)
        if matches is not None:
            for match in matches:
                attributeName = match[0]
                attributeValue = match[1]
                dict[attributeName] = attributeValue
        return dict



    # END class GraphvizLayoutPolicy
    pass



class PortPolicy(object):


    # TODO:
    # the following should be configurable
    # port size, currently (8,8)
    # min port margin, currently 5
    # port spacing, currently dynamically spaced out

    def __init__(self, contextManager):
        self.contextManager(contextManager)
        return

    def contextManager(self, value=None):
        if value is not None:
            self._manager = value
        return self._manager


    def setupDimensions(self, port):
        port.width = 8
        port.height = 8
        return



    # END class PortPolicy
    pass




import logging
import os
import uuid

import wx
import wx.xrc

import pomsets.definition as DefinitionModule

import pomsets_app.utils as AppUtilsModule

def manageHadoopConfiguration(contextManager, window):

    automaton = contextManager.automaton()
    
    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'hadoop frame.xrc'))

    import pomsets_app.gui.widget.hadoop as WidgetModule
    dialog = WidgetModule.ConfigDialog(res, window)

    configuration = automaton.hadoopConfigurations().get('localhost', None)

    shouldCreateNewConfig = False
    if configuration is None:
        configuration = {'hostname':'localhost'}
        shouldCreateNewConfig = True

    # first get the existing values from the automaton
    # populate the dialog
    hadoopHomeTextCtrl = dialog.getWidget('hadoopHomeTextCtrl')
    defaultHadoopHome = configuration.get('home', '')
    
    hadoopHomeTextCtrl.SetValue(defaultHadoopHome)
    
    relativeToHadoopHomeCheckBox = dialog.getWidget('relativeToHadoopHomeCheckBox')
    isRelativeToHadoopHome = configuration.get(
        'streaming jar is relative to home', True)
    relativeToHadoopHomeCheckBox.SetValue(isRelativeToHadoopHome)
    
    streamingJarTextCtrl = dialog.getWidget('streamingJarTextCtrl')
    defaultStreamingJar = ''
    if isRelativeToHadoopHome:
        defaultStreamingJar = os.path.join(
            'contrib', 'streaming',
            'hadoop-0.20.1-streaming.jar')
    else:
        defaultStreamingJar = os.path.join(
            defaultHadoopHome, 'contrib', 'streaming',
            'hadoop-0.20.1-streaming.jar')
        
    streamingJarTextCtrl.SetValue(
        configuration.get('streaming jar', defaultStreamingJar))
    
    try:
        if dialog.ShowModal() == wx.ID_OK:

            configuration['home'] = hadoopHomeTextCtrl.GetValue()
            
            # TODO:
            # handle the checkbox for whether the path
            # is relative to hadoop home
            # if it is, then prepend hadoop home
            isRelativeToHadoopHome = relativeToHadoopHomeCheckBox.GetValue()
            configuration['streaming jar is relative to home'] = isRelativeToHadoopHome
            configuration['streaming jar'] = streamingJarTextCtrl.GetValue()
            
            if shouldCreateNewConfig:
                logging.debug("need to implement when no configuration found")
                print "need to implement when no configuration found"
                automaton.addHadoopConfiguration(configuration)

            # save out to file
            configData = AppUtilsModule.createNewConfigDataObject()
            automaton.addConfigToSave(configData)
            AppUtilsModule.saveConfig(configData)
            pass
        pass
    except Exception, e:
        logging.error("errored on managing Hadoop configuration >> %s" % e)
        raise
    finally:
        dialog.Destroy()
    
    return

def manageEc2Credentials(contextManager, window):

    automaton = contextManager.automaton()

    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'euca2ools frame.xrc'))


    import pomsets_app.gui.widget.eucalyptus as WidgetModule
    dialog = WidgetModule.ConfigDialog(res, window)

    # first get the existing values from the automaton
    # and populate the dialog
    # valueKeyPair
    # valueIdentityFile (need a file browser)
    # valueImageId
    keyPairTextCtrl = dialog.getWidget('ValueKeyPair')
    identityFileTextCtrl = dialog.getWidget('ValueIdentityFile')
    textCtrlURL = dialog.getWidget('textCtrlURL')
    textCtrlAccessKey = dialog.getWidget('textCtrlAccessKey')
    textCtrlSecretKey = dialog.getWidget('textCtrlSecretKey')

    shouldCreateNew = False
    try:
        controllerValues = automaton.getCloudControllerCredentialsForAPI('euca2ools')
        keyPairTextCtrl.SetValue(controllerValues.get('user key pair', ''))
        identityFileTextCtrl.SetPath(controllerValues.get('identity file', ''))
        textCtrlURL.SetValue(controllerValues.get('url', ''))
        textCtrlAccessKey.SetValue(controllerValues.get('access key', ''))
        textCtrlSecretKey.SetValue(controllerValues.get('secret key', ''))

    except KeyError, e:
        # that means there hasnt been any defined
        shouldCreateNew = True
        pass

    try:
        if dialog.ShowModal() == wx.ID_OK:

            if shouldCreateNew:
                controllerValues = {}

            controllerValues['user key pair'] = keyPairTextCtrl.GetValue()
            controllerValues['identity file'] = identityFileTextCtrl.GetPath()

            controllerValues['url'] = textCtrlURL.GetValue()
            controllerValues['access key'] = textCtrlAccessKey.GetValue()
            controllerValues['secret key'] = textCtrlSecretKey.GetValue()

            if shouldCreateNew:
                automaton.addCloudControllerCredential(
                    'Eucalyptus', 'euca2ools', **controllerValues)
                pass

            # save out to file
            configData = AppUtilsModule.createNewConfigDataObject()
            automaton.addConfigToSave(configData)
            AppUtilsModule.saveConfig(configData)
            
            pass
        pass
    except Exception, e:
        logging.error("errored on managing EC2 credentials >> %s" % e)
        raise
    finally:
        dialog.Destroy()

    return



def editParameter(contextManager, window, parameterToEdit):
    
    automaton = contextManager.automaton()

    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'edit parameter dialog.xrc'))

    import pomsets_app.gui.widget.definition as WidgetModule
    dialog = WidgetModule.EditDefinitionParameterDialog(res, window)

    try:
        dialog.populate()
        dialog.setObjectToEdit(parameterToEdit)
        if dialog.ShowModal() == wx.ID_OK:

            # TODO:
            # modify the definition and save it
            logging.debug("should edit parameter")
            pass
        pass
    except Exception, e:
        logging.error("errored on editing parameter %s" % e)
        raise
    finally:
        dialog.Destroy()
        
        
    return


def createDefinition(contextManager, window):
    
    # autogenerate the name of the definition
    id = uuid.uuid4().hex
    name = '_'.join(['definition', id[:3]])
    definition = DefinitionModule.createShellProcessDefinition(
        name=name, inputParameters={}
    )
    definition.id(id)

    import pomsets.context as ContextModule
    context = ContextModule.Context()
    context.pomset(definition)
    context.isModified(False)
    contextManager.transientLibrary().addPomsetContext(context)

    return context

    
    
def editDefinition(contextManager, window, definitionToEdit):
    
    automaton = contextManager.automaton()

    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'edit definition dialog.xrc'))

    import pomsets_app.gui.widget.definition as WidgetModule
    dialog = WidgetModule.EditAtomicDefinitionDialog(res, window)
    dialog.contextManager = contextManager
    
    try:
        dialog.populate()
        # dialog.setObjectToEdit(definitionToEdit)
        contextToEdit = contextManager.getContextForDefinition(definitionToEdit)
        dialog.setObjectToEdit(contextToEdit)
    
        if dialog.ShowModal() == wx.ID_OK:

            # TODO:
            # modify the definition and save it
            logging.debug("should edit definition")
            pass
        pass
    except Exception, e:
        logging.error("errored on editing definition >> %s" % e)
        raise
    finally:
        dialog.Destroy()
        
        
    return


"""
def editDefinitionList(contextManager, window):
    
    
    # TODO:
    # either need to display a dialog box
    # and ask the user to create an active pomset
    # or don't allow the user to call this in the first place
    # which can be done by disabling the menu
    
    automaton = contextManager.automaton()

    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'definition list dialog.xrc'))

    dialog = WidgetModule.DefinitionListDialog(res, window)
    dialog.contextManager = contextManager
    
    try:
        dialog.populateListBox()
        if dialog.ShowModal() == wx.ID_OK:

            # TODO:
            # modify the definition and save it
            logging.debug("should edit definitions list")
            pass
        pass
    except Exception, e:
        logging.error("errored on editing definitions list" % e)
        raise
    finally:
        dialog.Destroy()
        
        
    return
"""



def addNode(contextManager, definition, name, position):

    node = contextManager.createNewNode(
        name=name, definitionToReference=definition)

    # TODO:
    # move elsewhere, so that we do not need to import 
    # the event module here
    import pomsets_app.gui.event as EventModule

    # post the event to update the GUI
    guiEvent = EventModule.NodeCreatedEvent(
        object=node,
        position=position,
        contextManager=contextManager)
    contextManager.postEvent(guiEvent)
    return


def selectDefinitionAndAddNode(contextManager, window, event):
    
    # display a dialog box to query the user for info
    #import pomsets_app.gui.widget.definition as WidgetModule
    #dialog = WidgetModule.SelectDefinitionDialog(
    #    window,
    #    WidgetModule.ID_DIALOG_SELECTDEFINITION, 
    #    "Select a definition")
    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'select definition.xrc'))
    import pomsets_app.gui.widget.definition as WidgetModule
    dialog = WidgetModule.SelectDefinitionDialog(res, window)

    # self.event is the event originating from the contextual menu
    # dialog.event = self.event
    dialog.contextManager = contextManager
    
    try:
        dialog.populate()
        if dialog.ShowModal() == wx.ID_OK:
            definition = dialog.selectedDefinition
            name = dialog.nodeName
            
            # need to create a new point object
            # because somehow the values get corrupted
            position = wx.Point(event.GetEventObject().event.GetX(),
                                event.GetEventObject().event.GetY())

            addNode(contextManager, definition, name, position)
        pass
    except Exception, e:
        logging.error("errored on selecting a definition >> %s" % e)
        raise
    finally:
        dialog.Destroy()
    
    return


def selectPathToSavePomsetDefinition(contextManager, window, title=None):

    dialog = wx.FileDialog(window,
                           title or "",
                           os.getcwd(), "", "*.pomset", 
                           wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)

    try:
        if dialog.ShowModal() == wx.ID_OK:
            path=dialog.GetPath()
            #contextManager.savePomsetAs(
            #    path=path
            #)
            return path
        pass
    except Exception, e:
        # TODO: 
        # should show an error dialog box here

        logging.error("errored on saving pomset >> %s" % e)
        raise e
    finally:
        dialog.Destroy()

    # reaches here if the user cancelled operation
    return


def selectPathToLoadPomsetDefinition(contextManager, window, title=None):
    
    # pull up the GUI to query the user for the pomset pickle to load
    # once the information is retrieved
    # should construct a command 
    # display a dialog box to query the user for info
    dialog = wx.FileDialog(window,
                           title or "",
                           os.getcwd(), "", "*.pomset", wx.OPEN)
    
    try:
        if dialog.ShowModal() == wx.ID_OK:
            path=dialog.GetPath()
            
            return path
        pass
    except Exception, e:
        logging.error("errored on loading pomset >> %s" % e)
        raise
    finally:
        dialog.Destroy()
        
    # we reach here if the user cancelled the operation
    return



def editReferenceDefinition(contextManager, window, 
                            definitionToEdit, parameterId=None):
    
    automaton = contextManager.automaton()

    resourcePath = contextManager.resourcePath()
    res =  wx.xrc.XmlResource(
        os.path.join(resourcePath, 'xrc', 'edit task dialog.xrc'))

    import pomsets_app.gui.widget.definition as WidgetModule
    dialog = WidgetModule.EditReferenceDefinitionDialog(res, window)
    dialog.contextManager = contextManager
    
    try:
        dialog.setObjectToEdit(definitionToEdit)
        if parameterId is not None:
            dialog.selectParameter(parameterId)
            dialog.populateParameterValues()
            
        if dialog.ShowModal() == wx.ID_OK:

            # TODO:
            # modify the definition and save it
            logging.debug("should edit definition")
            pass
        pass
    except Exception, e:
        logging.error("errored on editing definition >> %s" % e)
        raise
    finally:
        dialog.Destroy()
        
        
    return


def showAboutDialog(contextManager):

    frame = contextManager.app().GetTopWindow()

    # First we create and fill the info object
    from wx.lib.wordwrap import wordwrap

    info = wx.AboutDialogInfo()
    info.Name = "pomsets"
    info.Version = "0.1.0"
    info.Copyright = u"\u00A9 2010 michael j pan"
    info.Description = wordwrap(
        "pomsets is a workflow management program for your cloud.",
        250, wx.ClientDC(frame))
    info.WebSite = ("http://pomsets.org", "pomsets home page")
    info.Developers = [ "michael j pan" ]

    licenseText = "See http://pomsets.org/License for licensing options."
    info.License = wordwrap(licenseText, 250, wx.ClientDC(frame))

    # Then we call wx.AboutBox giving it that info object
    wx.AboutBox(info)
    
    return



import wx

import pomsets_app.gui.widget as WidgetModule

class ConfigDialog(WidgetModule.Dialog):

    XRC_NAME = 'HadoopDialog'
    WIDGET_BINDINGS = WidgetModule.Dialog.WIDGET_BINDINGS + []

    def __init__(self, xmlResource, parent=None):
        WidgetModule.Dialog.__init__(
            self, xmlResource.LoadDialog(parent, ConfigDialog.XRC_NAME))

        hadoopHomeTextCtrl = self.getWidget('hadoopHomeTextCtrl')
        hadoopHomeTextCtrl.SetMinSize(wx.Size(300, 25))
        streamingJarTextCtrl = self.getWidget('streamingJarTextCtrl')
        streamingJarTextCtrl.SetMinSize(wx.Size(300, 25))

        self.SetMinSize(wx.Size(325, 220))
        self.Fit()
        self.SetAutoLayout(True)
        return


    pass
